#include <conio.h>
#include <stdlib.h>
#include <math.h>

#include <pr.h>
#include <prgui.h>

#ifdef __3DFX__
#include <pr3dfx.h>
#include <glide.h>
#endif

#ifdef WIN32
#include <winutil.h>
#endif

#include <prdev.h>

/* Power Render Example 12

   Description:  Shows how to use the particle system

   Press the numbers 1 through 8 to create different emitters.
   After setting up the emitter and particle templates, each one of these
   effects is generated with a single function call.

   Use the mouse to rotate and move the camera.  Left and right buttons
   control the zoom factor.
*/

                                            
PR_VIEWPORT viewport;                  /* Our viewport structure */
PR_CAMERA *camera;                     /* One camera */
PR_LIGHTLIST userlights;               /* Holds a list of lights */
PR_DWORD animation_count;

#define TIME_DELAY 20

#define MAX_PARTICLE_SHAPES 16
PR_OBJECT *ParticleObjects[MAX_PARTICLE_SHAPES];
PR_ENTITY *ParticleEntities[MAX_PARTICLE_SHAPES];
PR_DWORD   ParticleMaterials[MAX_PARTICLE_SHAPES];

/* Emitter Templates */
#define FLARETRAIL_EMITTER 0
#define SPIRAL_EMITTER     1
#define SPARK_EMITTER      2
#define SMOKE_EMITTER      3
#define ASH_EMITTER        4
#define BOUNCEPUFF_EMITTER 5
#define EXPLOSION_EMITTER  6
#define RING_EMITTER       7

/* Particle Templates */
#define FLARETRAIL_PARTICLE 0
#define SPIRAL_PARTICLE     1
#define SPARK_PARTICLE      2
#define SMOKE_PARTICLE      3
#define ASH_PARTICLE        4
#define BOUNCEPUFF_PARTICLE 5
#define EXPLOSION_PARTICLE  6
#define RING_PARTICLE       7


void DrawParticle (PR_PARTICLE *particle)
{
  PR_TransformEntity (particle->Entity);
  PR_RenderEntity (particle->Entity);
}


void BouncePuffCollide (PR_EMITTER *emit)
{
PR_POINT dvector;
PR_POINT location;

  if (emit->Origin.y < 0)
    {
     emit->Origin.y = 0;
     emit->Velocity.y *= emit->BounceFactor.y;
     emit->Velocity.x += (rand () % 5) - 2;
     emit->Velocity.z += (rand () % 5) - 2;

     dvector.x = 0.0f;
     dvector.y = 0.0f;
     dvector.z = 0.0f;
     location.x = emit->Origin.x;
     location.y = emit->Origin.y + (rand () % 1000) * 0.01f;
     location.z = emit->Origin.z;
     PR_CreateEmitter (RING_EMITTER, &location, &dvector);
     PR_CreateEmitter (SPARK_EMITTER, &location, &dvector);
     PR_CreateEmitter (ASH_EMITTER, &location, &dvector);
    }
}



void CreateParticleMaterials (void)
{
PR_MATERIAL m;
PR_DWORD i;

  PR_SetTextureFormat (TEXTURE_P_8);

  /* Load the textures used by particles */
  PR_LoadTexture ("flare1.pcx");
  PR_LoadTexture ("flare2.pcx");
  PR_LoadTexture ("flare3.pcx");
  PR_LoadTexture ("smoke.pcx");

  /* Create some materials */

  /* Common values to all materials */
  PR_SetMaterialMethod (&m, T_LIGHTMAP_ADD);
  PR_SetMaterialColor (&m, 255, 255, 255);
  PR_SetMaterialShades (&m, 31);
  PR_SetMaterialEnvironmentMap (&m, 0);
  PR_SetMaterialMipMapState (&m, FALSE);
  PR_SetMaterialAlpha (&m, 255);

  PR_SetMaterialName (&m, "flare1");
  PR_SetMaterialTexture (&m, PR_FindTexture("flare1.pcx"));
  ParticleMaterials[0] = PR_AddMaterial (&m);

  PR_SetMaterialName (&m, "flare2");
  PR_SetMaterialTexture (&m, PR_FindTexture("flare2.pcx"));
  ParticleMaterials[1] = PR_AddMaterial (&m);

  PR_SetMaterialName (&m, "flare3");
  PR_SetMaterialTexture (&m, PR_FindTexture("flare3.pcx"));
  ParticleMaterials[2] = PR_AddMaterial (&m);

  PR_SetMaterialName (&m, "smoke");
  PR_SetMaterialTexture (&m, PR_FindTexture("smoke.pcx"));
  ParticleMaterials[3] = PR_AddMaterial (&m);

  ParticleMaterials[BOUNCEPUFF_PARTICLE] = ParticleMaterials[SMOKE_PARTICLE];
  ParticleMaterials[ASH_PARTICLE] = ParticleMaterials[SPIRAL_PARTICLE];

  /* Now create sprite objects and entities that use these materials */
  for (i = 0; i < 6; i++)
    {
     ParticleObjects[i] = PR_AllocSprite (ParticleMaterials[i]);
     ParticleEntities[i] = PR_CreateEntity (ParticleObjects[i], "particle");

     ParticleEntities[i]->userdata[0] = ParticleMaterials[i];
     /* This is required to let the particle system know which material to
        use for alpha fading. */
    }


  ParticleObjects[EXPLOSION_PARTICLE] = PR_LoadPRO ("explode.pro", LOAD_NORMAL);
  ParticleEntities[EXPLOSION_PARTICLE] = PR_CreateEntity (ParticleObjects[EXPLOSION_PARTICLE], "particle");
  ParticleEntities[EXPLOSION_PARTICLE]->userdata[0] = PR_FindMaterial ("explode");

  ParticleObjects[RING_PARTICLE] = PR_LoadPRO ("ring.pro", LOAD_NORMAL);
  ParticleEntities[RING_PARTICLE] = PR_CreateEntity (ParticleObjects[RING_PARTICLE], "particle");
  ParticleEntities[RING_PARTICLE]->userdata[0] = PR_FindMaterial ("ring");
} 


void InitializeParticleTemplates (void)
{
PR_MATRIX baserot;
PR_MATRIX angular;
PR_POINT distance;
PR_POINT bouncefactor;


  PR_AllocParticles (2000, 20);
  PR_AllocEmitters (100, 20);

  /* Flaretrail */
  PR_InitializeParticleType (FLARETRAIL_PARTICLE);
  PR_SetParticleAlpha (FLARETRAIL_PARTICLE, 255, -15);
  PR_SetParticleScale (FLARETRAIL_PARTICLE, 1.0f, -0.033f);
  PR_SetParticleEntity (FLARETRAIL_PARTICLE, ParticleEntities[FLARETRAIL_PARTICLE]);
  PR_SetParticleDrawProc (FLARETRAIL_PARTICLE, DrawParticle);

  PR_InitializeEmitterType (FLARETRAIL_EMITTER);
  PR_SetEmitterMovement (FLARETRAIL_EMITTER, 1);
  PR_SetEmitterTimeLimit (FLARETRAIL_EMITTER, 200);
  PR_SetEmitterDelay (FLARETRAIL_EMITTER, 2);
  PR_SetEmitterGravity (FLARETRAIL_EMITTER, -0.03);
  PR_SetEmitterParticleType (FLARETRAIL_EMITTER, FLARETRAIL_PARTICLE);


  /* Spiral trail */
  PR_MatrixIdentity (baserot);
  PR_MatrixRotate (angular, 0, 4, 0);
  distance.x = 0;
  distance.y = 0;
  distance.z = 75;
  PR_InitializeParticleType (SPIRAL_PARTICLE);
  PR_SetParticleAlpha (SPIRAL_PARTICLE, 255, -4);
  PR_SetParticleMovement (SPIRAL_PARTICLE, 1);
  PR_SetParticleGravity (SPIRAL_PARTICLE, -0.013f);
  PR_SetParticleDrag (SPIRAL_PARTICLE, 0.98f);
  PR_SetParticleScale (SPIRAL_PARTICLE, 0.1f, 0.0f);
  PR_SetParticleEntity (SPIRAL_PARTICLE, ParticleEntities[SPIRAL_PARTICLE]);
  PR_SetParticleDrawProc (SPIRAL_PARTICLE, DrawParticle);

  PR_InitializeEmitterType (SPIRAL_EMITTER);
  PR_SetEmitterMovement (SPIRAL_EMITTER, 1);
  PR_SetEmitterTimeLimit (SPIRAL_EMITTER, 150);
  PR_SetEmitterDelay (SPIRAL_EMITTER, 2);
  PR_SetEmitterParticleType (SPIRAL_EMITTER, SPIRAL_PARTICLE);
  PR_SetEmitterRotation (SPIRAL_EMITTER, baserot, angular, &distance);
  PR_SetEmitterRandomDistance (SPIRAL_EMITTER, 2);
  PR_SetEmitterRandomVelocity (SPIRAL_EMITTER, 1);


  /* Spark Explosion */
  PR_InitializeParticleType (SPARK_PARTICLE);
  PR_SetParticleAlpha (SPARK_PARTICLE, 255, -4);
  PR_SetParticleMovement (SPARK_PARTICLE, 1);
  PR_SetParticleGravity (SPARK_PARTICLE, -0.053f);
  PR_SetParticleScale (SPARK_PARTICLE, 0.3f, 0.0f);
  PR_SetParticleEntity (SPARK_PARTICLE, ParticleEntities[SPARK_PARTICLE]);
  PR_SetParticleDrawProc (SPARK_PARTICLE, DrawParticle);

  PR_InitializeEmitterType (SPARK_EMITTER);
  PR_SetEmitterTimeLimit (SPARK_EMITTER, 1);
  PR_SetEmitterNumParticles (SPARK_EMITTER, 15);
  PR_SetEmitterParticleType (SPARK_EMITTER, SPARK_PARTICLE);
  PR_SetEmitterRandomVelocity (SPARK_EMITTER, 5);


  /* Ash Explosion */
  PR_InitializeParticleType (ASH_PARTICLE);
  PR_SetParticleAlpha (ASH_PARTICLE, 255, -4);
  PR_SetParticleMovement (ASH_PARTICLE, 1);
  PR_SetParticleGravity (ASH_PARTICLE, -0.093f);
  PR_SetParticleScale (ASH_PARTICLE, 0.2f, 0.0f);
  PR_SetParticleEntity (ASH_PARTICLE, ParticleEntities[ASH_PARTICLE]);
  PR_SetParticleDrawProc (ASH_PARTICLE, DrawParticle);

  PR_InitializeEmitterType (ASH_EMITTER);
  PR_SetEmitterTimeLimit (ASH_EMITTER, 1);
  PR_SetEmitterNumParticles (ASH_EMITTER, 10);
  PR_SetEmitterParticleType (ASH_EMITTER, ASH_PARTICLE);
  PR_SetEmitterRandomVelocity (ASH_EMITTER, 1);

  /* Rising Smoke */
  PR_InitializeParticleType (SMOKE_PARTICLE);
  PR_SetParticleAlpha (SMOKE_PARTICLE, 255, -2);
  PR_SetParticleScale (SMOKE_PARTICLE, 0.1f, 0.013f);
  PR_SetParticleGravity (SMOKE_PARTICLE, 0.023f);  /* positive goes up */
  PR_SetParticleMovement (SMOKE_PARTICLE, 1);
  PR_SetParticleEntity (SMOKE_PARTICLE, ParticleEntities[SMOKE_PARTICLE]);
  PR_SetParticleDrawProc (SMOKE_PARTICLE, DrawParticle);

  PR_InitializeEmitterType (SMOKE_EMITTER);
  PR_SetEmitterTimeLimit (SMOKE_EMITTER, 1);
  PR_SetEmitterParticleType (SMOKE_EMITTER, SMOKE_PARTICLE);


  /* Bouncing smoke puff */
  bouncefactor.x = 1.0f;
  bouncefactor.y = -0.95f;
  bouncefactor.z = 1.0f;
  PR_MatrixIdentity (baserot);
  PR_MatrixRotate (angular, 5, 10, 20);
  distance.x = 0;
  distance.y = 0;
  distance.z = 20;
  PR_InitializeParticleType (BOUNCEPUFF_PARTICLE);
  PR_SetParticleAlpha (BOUNCEPUFF_PARTICLE, 255, -2);
  PR_SetParticleScale (BOUNCEPUFF_PARTICLE, 0.1f, 0.003f);
  PR_SetParticleEntity (BOUNCEPUFF_PARTICLE, ParticleEntities[BOUNCEPUFF_PARTICLE]);
  PR_SetParticleDrawProc (BOUNCEPUFF_PARTICLE, DrawParticle);

  PR_InitializeEmitterType (BOUNCEPUFF_EMITTER);
  PR_SetEmitterTimeLimit (BOUNCEPUFF_EMITTER, 500);
  PR_SetEmitterParticleType (BOUNCEPUFF_EMITTER, BOUNCEPUFF_PARTICLE);
  PR_SetEmitterBounceParameters (BOUNCEPUFF_EMITTER, &bouncefactor, 20);
  PR_SetEmitterRotation (BOUNCEPUFF_EMITTER, baserot, angular, &distance);
  PR_SetEmitterGravity (BOUNCEPUFF_EMITTER, -0.23);
  PR_SetEmitterMovement (BOUNCEPUFF_EMITTER, 1);
  PR_SetEmitterCollisionProc (BOUNCEPUFF_EMITTER, BouncePuffCollide);
  PR_SetEmitterDelay (BOUNCEPUFF_EMITTER, 1);


  /* 3D Mesh Explosion */
  PR_InitializeParticleType (EXPLOSION_PARTICLE);
  PR_SetParticleAlpha (EXPLOSION_PARTICLE, 255, -4);
  PR_SetParticleScale (EXPLOSION_PARTICLE, 0.3f, 0.023f);
  PR_SetParticleEntity (EXPLOSION_PARTICLE, ParticleEntities[EXPLOSION_PARTICLE]);
  PR_SetParticleDrawProc (EXPLOSION_PARTICLE, DrawParticle);

  PR_InitializeEmitterType (EXPLOSION_EMITTER);
  PR_SetEmitterTimeLimit (EXPLOSION_EMITTER, 1);
  PR_SetEmitterParticleType (EXPLOSION_EMITTER, EXPLOSION_PARTICLE);


  /* Ring Mesh Explosion */
  PR_InitializeParticleType (RING_PARTICLE);
  PR_SetParticleAlpha (RING_PARTICLE, 255, -4);
  PR_SetParticleScale (RING_PARTICLE, 0.05f, 0.043f);
  PR_SetParticleEntity (RING_PARTICLE, ParticleEntities[RING_PARTICLE]);
  PR_SetParticleDrawProc (RING_PARTICLE, DrawParticle);

  PR_InitializeEmitterType (RING_EMITTER);
  PR_SetEmitterTimeLimit (RING_EMITTER, 1);
  PR_SetEmitterParticleType (RING_EMITTER, RING_PARTICLE);

}







void animation_timer (void)
{
 animation_count++;
}

void main (void)
{
PR_DWORD vwidth, vheight;
PR_UDWORD BLACK;                       /* Color index/value for black */
PR_DWORD move_count;
PR_POINT dvector, location;
PR_DWORD reload_delay = 0;
PR_REAL camrot = 130;
PR_REAL camhgt = 100;
PR_REAL camdst = 400;

  ChooseDevice ();

  PR_Initialize (5000);                /* Maximum 2000 triangles per frame */

  if ((device == DEVICE_VGA) || (device == DEVICE_SVGA))
    {
     #ifndef WIN32
       wsetmode (3);                        /* Go back to text mode */
     #endif

     PR_FatalError ("This example requires a 3D card", "EX12");
    }

  InitializeDevices ();

  PR_AllocMaterials (64);              /* Allocate the materials, */
  PR_AllocTextures (32);               /* textures, */
  PR_AllocShadeTables (2);             /* and shade tables */

  vwidth = PR_VideoModes[0].width;     /* Initialize the video mode */
  vheight = PR_VideoModes[0].height;
  PR_SetMode (vwidth, vheight, 60);

  PR_DepthBufferFunction (PR_CMP_ALWAYS);


  PR_OpenViewport (&viewport, 0, 0, vwidth-1, vheight-1, VIEW_PLAIN);
  PR_SetViewport (&viewport);          /* Open a full screen viewport */

  CreateParticleMaterials ();
  InitializeParticleTemplates();


  /* Initialize the camera */
  camera = PR_AllocCamera ();
  PR_InitializeCamera (camera);
  PR_PositionCameraSource (camera, 0, 100, 400);
  PR_PositionCameraTarget (camera, 0, 0, 0);
  PR_SetCameraMode (camera, CAMFLAG_AIM_TARGET); 
  PR_SetActiveCamera (camera);

 /* Initialize the light */
  PR_AllocLights (&scenelights, 1);
  PR_AllocLights (&userlights, 1);
  PR_SetLightPosition (&userlights, 0, 0, 0, 400);
  PR_SetLightOn (&userlights, 0);
  PR_SetLightType (&userlights, 0, DIRECTIONAL_LIGHT);
  PR_SetLightColor (&userlights, 0, 1.0, 1.0, 1.0);
  PR_SetLightStrength (&userlights, 0, 1.0);
  userlights.NumLights = 1;     /* This sets how many lights are actually used */

  PR_OpenScreen (PR_BACKBUFFER);
  BLACK = PRGFX_MakeColor (0, 0, 0);

  installkbd ();
#ifndef WIN32
  minit ();
#endif

  animation_count = 0;
  winittimer ();
  wstarttimer (animation_timer, TICKS (60));

  msetxy (160, 100);

  /* main program loop */
  while (!kbdon[KEY_ESC])
    {
     #ifdef WIN32
       UpdateMessages ();
     #endif      

     #if defined (MSDD) || defined (WTDD) || defined (MSGL) || defined (WTGL)
       if (PR_LostSurface)
         PR_RestoreAll ();
     #endif

     PR_OpenScreen (PR_BACKBUFFER);

     PRGFX_SetColor (BLACK);
     PRGFX_ClearScreen ();             /* Clear the last frame */

     PR_NewFrame ();                   /* Begin a new frame */

     camrot += mouse.mx - 160;
     camhgt += mouse.my - 100;
     if (mouse.but == 1)
       camdst += 20;
     if ((mouse.but == 2) && (camdst > 20))
       camdst -= 20;
     msetxy (160, 100);
     PR_PositionCameraSource (camera, cos (camrot * 0.01) * camdst, camhgt,
                                      sin (camrot * 0.01) * camdst);
     PR_SetActiveCamera (camera);

     PR_TransformLights (&userlights);

     move_count = animation_count;
     animation_count = 0;

     if (reload_delay == 0)
       {
        /* blue flare trails */
        if (kbdon[KEY_1])
          {
           dvector.x = 0.0f;
           dvector.y = 0.0f;
           dvector.z = -30.0f;
           location.x = 200.0f;
           location.y = 0.0f;
           location.z = 400.0f;
           PR_CreateEmitter (0, &location, &dvector);

           location.x = -200.0f;
           PR_CreateEmitter (FLARETRAIL_EMITTER, &location, &dvector);
  
           reload_delay = TIME_DELAY;
          }

        /* spiral */
        if (kbdon[KEY_2])
          {
           dvector.x = 0.0f;
           dvector.y = 0.3f;
           dvector.z = 0.0f;
           location.x = 0.0f;
           location.y = -10.0f;
           location.z = 100.0f;
           PR_CreateEmitter (SPIRAL_EMITTER, &location, &dvector);
           reload_delay = TIME_DELAY;
          }


        /* sparks */
        if (kbdon[KEY_3])
          {
           dvector.x = 0.0f;
           dvector.y = 0.0f;
           dvector.z = 0.0f;
           location.x = 0.0f;
           location.y = 70.0f;
           location.z = 200.0f;
           PR_CreateEmitter (SPARK_EMITTER, &location, &dvector);
           reload_delay = TIME_DELAY;
          }

        /* smoke */
        if (kbdon[KEY_4])
          {
           dvector.x = 0.0f;
           dvector.y = 0.0f;
           dvector.z = 0.0f;
           location.x = 0.0f;
           location.y = 0.0f;
           location.z = 0.0f;
           PR_CreateEmitter (SMOKE_EMITTER, &location, &dvector);
           reload_delay = TIME_DELAY;
          }


        /* ashes */
        if (kbdon[KEY_5])
          {
           dvector.x = 0.0f;
           dvector.y = 0.0f;
           dvector.z = 0.0f;
           location.x = 0.0f;
           location.y = 0.0f;
           location.z = 0.0f;
           PR_CreateEmitter (ASH_EMITTER, &location, &dvector);
           reload_delay = TIME_DELAY;
          }

        /* 3D Mesh explosion */
        if (kbdon[KEY_6])
          {
           dvector.x = 0.0f;
           dvector.y = 0.0f;
           dvector.z = 0.0f;
           location.x = 0.0f;
           location.y = 0.0f;
           location.z = 0.0f;
           PR_CreateEmitter (EXPLOSION_EMITTER, &location, &dvector);
           reload_delay = TIME_DELAY;
          }


        /* Ring Mesh explosion */
        if (kbdon[KEY_7])
          {
           dvector.x = 0.0f;
           dvector.y = 0.0f;
           dvector.z = 0.0f;
           location.x = 0.0f;
           location.y = 0.0f;
           location.z = 0.0f;
           PR_CreateEmitter (RING_EMITTER, &location, &dvector);
           reload_delay = TIME_DELAY;
          }

        /* Bouncing Smoke Puff */
        if (kbdon[KEY_8])
          {
           dvector.x = (rand () % 5) - 2;
           dvector.y = 0.0f;
           dvector.z = (rand () % 5) - 2;
           location.x = 0.0f;
           location.y = 100.0f;
           location.z = 0.0f;
           PR_CreateEmitter (BOUNCEPUFF_EMITTER, &location, &dvector);
           reload_delay = TIME_DELAY;
          }
       }
     else
       {
        if (reload_delay > 0)
          reload_delay--;
       }

     PR_UpdateEmitters (move_count);
     PR_UpdateParticles (move_count);

     /* Make the 3D explosion mesh rotate */
     PR_RotateEntity (ParticleEntities[EXPLOSION_PARTICLE], 0.5f, 0.7f, -0.3f);
     PR_RotateEntity (ParticleEntities[RING_PARTICLE], 0.0f, 2.0f, 0.0f);
     PR_RotateSegment (ParticleEntities[SMOKE_PARTICLE], 0, 0.0f, 0.0f, -0.8f);

     PR_RenderFrame ();                /* Draw the scene */

     PR_Flip (1);                      /* Show the background page */
    }

  /* Free all allocated memory */
  PR_FreeCamera (camera);
  PR_CloseViewport (&viewport);
  PR_DeleteAllShadeTables ();
  PR_DeleteAllTextures ();
  PR_DeleteAllMaterials ();

  PR_ShutDown ();

  uninstallkbd ();
  wstoptimer ();
  wdonetimer ();
  mdeinit ();

#ifndef WIN32
  wsetmode (3);                        /* Go back to text mode */
#endif
}

