#include <conio.h>
#include <stdlib.h>

#include <pr.h>
#include <prgui.h>

#ifdef __3DFX__
#include <pr3dfx.h>
#include <glide.h>
#endif

#ifdef WIN32
#include <winutil.h>
#endif

#include <prdev.h>

/* Power Render Example 13

   Description:  This is used to test all of the rendering methods.

   Use +/- to change the rendering methods by 1.  PGUP and PGDN will
   change by 10.

   A/Z moves the camera.


  If the rendering method has precalculated lighting, it will generate
  random light values at each vertex of every face.


  Shadetables used:
    Gouraud (gour.tab) - regular gouraud table with 32 shades
    Translucent (trans.tab) - translucent table with 0.5 foreground and background
    lightmap (light.tab) - translucent table with 1.0 foreground and background (adds them together)
    Shadow (src.tab) - Gouraud table with 2 shades, start and end at 0.5.  This is used for the source_transform method.
                       With this table it darkens the source, which could be used for shadows.

*/

                                            
PR_VIEWPORT viewport;                  /* Our viewport structure */
PR_CAMERA *camera;                     /* One camera */
PR_OBJECT *object_shape;               /* The shape loaded */
PR_ENTITY *entity;                     /* A single entity of the shape */
PR_LIGHTLIST userlights;               /* Holds a list of lights */
PR_DWORD animation_count;

PR_REAL camdist = 600;

PR_DWORD backtex;
PR_DWORD matnum;
PR_DWORD currentmode = 1;

PR_DWORD gourtab, transtab, srctab, lighttab;


void animation_timer (void)
{
 animation_count++;
}


void nokey (PR_DWORD keynum)
/* Waits for a specific key to be released */
{
PR_DWORD dummy = 0;

 while (kbdon[keynum])
   {
    #ifdef WIN32
      UpdateMessages ();
    #endif      
    dummy++;
   }
}


void RandomLight (PR_OBJECT *obj)
/* Sets random precalculated light values, from 0-31 */
{
PR_SEGMENT *seg;
PR_FACE *face;

PR_DWORD segnum, facenum, vertnum;
  
  seg = obj->segment_list;
  for (segnum = 0; segnum < obj->num_segments; segnum++)
    {
     face = seg->face_list;
     for (facenum = 0; facenum < seg->num_faces; facenum++)
       {
        for (vertnum = 0; vertnum < 3; vertnum++)
          {
           if (PR_Settings.Hardware)
              *(PR_REAL *)&face->face_data.col[vertnum] = 
                    (rand () % 256) / 255.0f;
           else
             face->face_data.col[vertnum] = (rand () % 32) << 16;
          }
        face++;
       }

     seg++;
    }
}



void RandomRGB (PR_OBJECT *obj)
/* Sets random precalculated RGB values */
{
PR_SEGMENT *seg;
PR_FACE *face;
PR_DWORD *rgb;
PR_DWORD r, g, b;

PR_DWORD segnum, facenum, vertnum;
  
  seg = obj->segment_list;
  for (segnum = 0; segnum < obj->num_segments; segnum++)
    {
     face = seg->face_list;
     for (facenum = 0; facenum < seg->num_faces; facenum++)
       {
        for (vertnum = 0; vertnum < 3; vertnum++)
          {
           rgb = (PR_DWORD *)&face->face_data.col[vertnum];

           r = rand () % 256;
           g = rand () % 256;
           b = rand () % 256;
           *rgb = r + (g << 8) + (b << 16);
          }


        face++;
       }

     seg++;
    }
}



void ClearLight (PR_OBJECT *obj)
/* Clears precalculated ramp lighting to 0 */
{
PR_SEGMENT *seg;
PR_FACE *face;

PR_DWORD segnum, facenum, vertnum;
  
  seg = obj->segment_list;
  for (segnum = 0; segnum < obj->num_segments; segnum++)
    {
     face = seg->face_list;
     for (facenum = 0; facenum < seg->num_faces; facenum++)
       {
        for (vertnum = 0; vertnum < 3; vertnum++)
          {
           if (PR_Settings.Hardware)
              *(PR_REAL *)&face->face_data.col[vertnum] = 0.0f;
           else
             face->face_data.col[vertnum] = 0;
          }
        face++;
       }

     seg++;
    }
}


void CreateMaterial (void)
{
  PR_ObjectMaterialList[matnum].render_method = currentmode;

  if ((device == DEVICE_SVGA) || (device == DEVICE_VGA))
    {
     PR_ObjectMaterialList[matnum].shadetable = gourtab;

     /* uses the appropriate shade table for software rendering */
     if (
        ((currentmode >= T_LTRANSLUCENT_TEXTURED) && (currentmode <= T_XWTRANSLUCENT_TEXTURED)) ||
        ((currentmode >= T_PTRANSLUCENT_TEXTURED) && (currentmode <= T_PXWTRANSLUCENT_TEXTURED)) 
        )
        PR_ObjectMaterialList[matnum].shadetable = transtab;

     if (currentmode == T_SOURCETRANSFORM)
       PR_ObjectMaterialList[matnum].shadetable = srctab;
 
     if (currentmode == T_LIGHTMAP_ADD)
       PR_ObjectMaterialList[matnum].shadetable = lighttab;
    }


  if (((currentmode >= T_LCFLAT_TEXTURED) && (currentmode <= T_XWCFLAT_TEXTURED)) ||
      ((currentmode >= T_LCGOURAUD_TEXTURED) && (currentmode <= T_XWCGOURAUD_TEXTURED8)) ||
      ((currentmode >= T_PCFLAT_TEXTURED) && (currentmode <= T_PXWCFLAT_TEXTURED)) ||
      ((currentmode >= T_PCGOURAUD_TEXTURED8) && (currentmode <= T_PXWCGOURAUD_TEXTURED8)) ||
      (currentmode == T_CFLAT) ||
      (currentmode == T_CGOURAUD))
    RandomLight (object_shape);
  else
    ClearLight (object_shape);

  if (((currentmode >= T_RGB_GOURAUD) && (currentmode <= T_RGB_TEXTURED_DYN)) ||
      ((currentmode >= T_FOG_RGB_GOURAUD) && (currentmode <= T_FOG_RGB_TEXTURED_DYN)))
    RandomRGB (object_shape);
}




void main (void)
{
PR_DWORD vwidth, vheight;
PR_REAL cenx, ceny, cenz;
PR_UDWORD BLACK;                       /* Color index/value for black */
PR_DWORD move_count;
PR_DWORD updatematerial = 1;


  ChooseDevice ();

  PR_Initialize (1000);                /* Maximum 1000 triangles per frame */

  InitializeDevices ();

  PR_AllocMaterials (32);              /* Allocate the materials, */
  PR_AllocTextures (32);               /* textures, */
  PR_AllocShadeTables (5);             /* and shade tables */

  vwidth = PR_VideoModes[0].width;     /* Initialize the video mode */
  vheight = PR_VideoModes[0].height;
  PR_SetMode (vwidth, vheight, 60);

  PR_Settings.HardwareMipmaps = MIPMAP_ALL_LEVELS;

  PR_OpenViewport (&viewport, 0, 0, vwidth-1, vheight-1, VIEW_PLAIN);
  PR_SetViewport (&viewport);          /* Open a full screen viewport */

  backtex  = PR_LoadTexture ("back.pcx");

  gourtab  = PR_LoadTable ("gour.tab");
  transtab = PR_LoadTable ("trans.tab");
  srctab   = PR_LoadTable ("src.tab");
  lighttab = PR_LoadTable ("light.tab");

  PR_SetTextureFormat (TEXTURE_RGB_565);
//  PR_SetTextureFormat (TEXTURE_XRAY);
//  PR_SetTextureFormat (TEXTURE_P_8);

  /* Load the object */
  object_shape = PR_LoadPRO ("box.pro", LOAD_NORMAL);
  PR_CenterObject (object_shape, &cenx, &ceny, &cenz);

  matnum = PR_FindMaterial ("testmat");
  if (matnum < 0)
    PR_FatalError ("Cannot find test material", "ex13");

#if !defined (MSGLIDE) && !defined (WTGLIDE) && !defined (MSGL) && !defined (WTGL)
  /* Set the palette from the textures loaded */
  wsetpalette (0, 255, global_palette);
#endif

  /* Create the main entity */
  entity = PR_CreateEntity (object_shape, "TEST ENTITY");


  /* Initialize the camera */
  camera = PR_AllocCamera ();
  PR_InitializeCamera (camera);
  PR_PositionCameraSource (camera, 0, 0, 400);
  PR_PositionCameraTarget (camera, 0, 0, 0);
  PR_SetCameraMode (camera, CAMFLAG_AIM_TARGET); 
  PR_SetActiveCamera (camera);

 /* Initialize the light */
  PR_AllocLights (&scenelights, 2);
  PR_AllocLights (&userlights, 2);
  PR_SetLightPosition (&userlights, 0, -800, 0, camdist);
  PR_SetLightOn (&userlights, 0);
  PR_SetLightType (&userlights, 0, DIRECTIONAL_LIGHT);
  PR_SetLightColor (&userlights, 0, 1.0, 0.25, 0.0);
  PR_SetLightStrength (&userlights, 0, 1.0);

  PR_SetLightPosition (&userlights, 1, 800, 0, camdist);
  PR_SetLightOn (&userlights, 1);
  PR_SetLightType (&userlights, 1, DIRECTIONAL_LIGHT);
  PR_SetLightColor (&userlights, 1, 0.0, 0.25, 1.0);
  PR_SetLightStrength (&userlights, 1, 1.0);

  userlights.NumLights = 2;     /* This sets how many lights are actually used */

  PR_OpenScreen (PR_BACKBUFFER);
  BLACK = PRGFX_MakeColor (0, 0, 0);

  installkbd ();
  animation_count = 0;
  winittimer ();
  wstarttimer (animation_timer, TICKS (60));

  gui_font = NULL;
  PRGFX_SetTextForeground (PRGFX_MakeColor (63, 63, 63));
  PRGFX_SetTextBackground (BLACK);
  PRGFX_SetTextTransparent (TEXTFGBG);

  PR_SetFogRange (500, 1500);

  PR_DepthBufferMode (PR_DEPTHBUFFER_DISABLE);

  /* main program loop */
  while (!kbdon[KEY_ESC])
    {
     #ifdef WIN32
       UpdateMessages ();
     #endif      

     #if defined (MSDD) || defined (WTDD) || defined (MSGL) || defined (WTGL)
       if (PR_LostSurface)
         PR_RestoreAll ();
     #endif

     PR_OpenScreen (PR_BACKBUFFER);

     PRGFX_SetColor (BLACK);
     PRGFX_ClearScreen ();             /* Clear the last frame */


     if ((kbdon[KEY_A]) && (camdist > 20))
        camdist -= 5;
     if (kbdon[KEY_Z])
        camdist += 5;
     PR_PositionCameraSource (camera, 0, 0, camdist);
     PR_SetActiveCamera (camera);

     if (kbdon[KEY_MINUS])
       {
        currentmode--;
        nokey (KEY_MINUS);
        updatematerial = 1;
       }
     if (kbdon[KEY_EQUALS])
       {
        currentmode++;
        nokey (KEY_EQUALS);
        updatematerial = 1;
       }
     if (kbdon[KEY_PGUP])
       {
        currentmode -= 10;
        nokey (KEY_PGUP);
        updatematerial = 1;
       }
     if (kbdon[KEY_PGDN])
       {
        currentmode += 10;
        nokey (KEY_PGDN);
        updatematerial = 1;
       }

     if (currentmode < 1)
       currentmode = 1;

     if (currentmode > MAXMETHOD)
       currentmode = MAXMETHOD;

     if ((device == DEVICE_SVGA) || (device == DEVICE_VGA))
       {
        if (currentmode > T_LIGHTMAP_ADD)
          currentmode = T_LIGHTMAP_ADD;
       }

     if (updatematerial)
       {
        CreateMaterial ();
        updatematerial = 0;
       }


     PR_NewFrame ();                   /* Begin a new frame */

     PR_TransformLights (&userlights);

     move_count = animation_count;
     animation_count = 0;


     PR_SetConstantZ (32767);
     PRGFX_ResizeTexture (0, 0, vwidth-1, vheight-1, backtex, NORMAL);

     PR_RotateEntity (entity, 0.3f, 0.2f, 0.1f);
     PR_TransformEntity (entity);      /* Transform the entity */
     PR_RenderEntity (entity);         /* Render the entity */

     PR_RenderFrame ();                /* Draw the scene */

     PRGUI_printf (10, 10, "Render Method: %i (%s)", currentmode, PR_ReturnMethodName (currentmode));

     PR_Flip (1);                      /* Show the background page */
    }

  /* Free all allocated memory */
  PR_FreeCamera (camera);
  PR_FreeEntity (entity);
  PR_FreeObject (object_shape);
  PR_CloseViewport (&viewport);
  PR_DeleteAllShadeTables ();
  PR_DeleteAllTextures ();
  PR_DeleteAllMaterials ();

  PR_ShutDown ();

  uninstallkbd ();
  wstoptimer ();
  wdonetimer ();

#ifndef WIN32
  wsetmode (3);                        /* Go back to text mode */
#endif
}

