#include <conio.h>
#include <stdlib.h>
#include <string.h>

#include <pr.h>
#include <prgui.h>

#ifdef __3DFX__
#include <pr3dfx.h>
#include <glide.h>
#endif

#ifdef WIN32
#include <winutil.h>
#endif

#include <prdev.h>

/* Power Render Example 14

   Description:  Demonstrates drawing on textures
                 
*/

                                            
PR_VIEWPORT viewport;                  /* Our viewport structure */
PR_CAMERA *camera;                     /* One camera */
PR_OBJECT *object_shape;               /* The shape loaded */
PR_ENTITY *entity;                     /* A single entity of the shape */
PR_LIGHTLIST userlights;               /* Holds a list of lights */
PR_DWORD animation_count;

void animation_timer (void)
{
 animation_count++;
}

unsigned char *OpenTexture (PR_DWORD texnum, PR_DWORD *pitch)
{
unsigned char *bits;

#if defined (MSDD) || defined (WTDD)
  bits = wgetblocksurface (PR_WorldTextures[texnum].image, pitch);
#else
  bits = PR_WorldTextures[texnum].image;
  *pitch = wgetblockwidth (bits);
  bits += 4;
#endif

  return bits;
}


void CloseTexture (PR_DWORD texnum)
{
#if defined (MSDD) || defined (WTDD)
  wunlocksurface (PR_WorldTextures[texnum].image);
#endif
}


void DrawOnTexture (unsigned char *bits, PR_DWORD width, PR_DWORD height)
{
PR_DWORD i, x, y;

  for (i = 0; i < 20; i++)
    {
     x = rand () % (width / 2);
     y = rand () % (height / 2);
     bits[y*width+x] = rand () % 255;
    }
}


void CyclePalette (PR_DWORD texnum, unsigned char start, unsigned char finish, 
                   PR_DWORD dir, color *pal)
/*  Rotate the colors in a direction */
{
  color temp;
  unsigned char *dst;
  unsigned char *src;
  short t;
  
  
  if  (start > finish)   /* Make sure start color # < end color # */
    {
     t = start;
     start = finish;
     finish = t;
    }

  if (dir == 0)                       /* Rotating up */
    {
     temp = pal[start];

     dst = (unsigned char *)&pal[start];
     src = (unsigned char *)&pal[start+1];

     /* Going to move everything up 1 */
     memmove (dst, src, abs (start - finish + 1) * 3);

     pal[finish - 1] = temp;
     /* Store first color at end */
    }
  else                                 /* Rotating down */
    {
     temp = pal[finish];

     src = (unsigned char *)&pal[start];
     dst = (unsigned char *)&pal[start+1];
 
     /* Going to move everything down 1 */
     memmove (dst, src, abs (start - finish) * 3);
    
     pal[start] = temp;
     /* Store last color in position 1 */
    }
}


void UpdateTexture (PR_DWORD texnum)
{
char *bits;
PR_DWORD pitch;

  bits = OpenTexture (texnum, &pitch);
  DrawOnTexture (bits, pitch, PR_WorldTextures[texnum].height);
  CloseTexture (texnum);

  if ((device == DEVICE_VGA) || (device == DEVICE_SVGA))
    {
     #if !defined(MSGLIDE) && !defined (WTGLIDE) && !defined (MSGL) && !defined (WTGL)
     CyclePalette (texnum, 1, 64, 0, global_palette);
     wsetpalette (1, 64, global_palette);
         #endif
    }
  else
    CyclePalette (texnum, 1, 64, 0, PR_WorldTextures[texnum].pal);

  PR_UpdateTexture (texnum, TRUE);
}



void main (void)
{
PR_DWORD vwidth, vheight;
PR_REAL cenx, ceny, cenz;
PR_UDWORD BLACK;                       /* Color index/value for black */
PR_DWORD move_count;

  ChooseDevice ();

  PR_Initialize (1000);                /* Maximum 1000 triangles per frame */

  InitializeDevices ();

  PR_AllocMaterials (32);              /* Allocate the materials, */
  PR_AllocTextures (32);               /* textures, */
  PR_AllocShadeTables (2);             /* and shade tables */

  vwidth = PR_VideoModes[0].width;     /* Initialize the video mode */
  vheight = PR_VideoModes[0].height;
  PR_SetMode (vwidth, vheight, 60);

  PR_OpenViewport (&viewport, 0, 0, vwidth-1, vheight-1, VIEW_PLAIN);
  PR_SetViewport (&viewport);          /* Open a full screen viewport */

  PR_SetTextureFormat (TEXTURE_P_8);

  /* Load the object */
  object_shape = PR_LoadPRO ("14.pro", LOAD_NORMAL);
  if (object_shape == NULL)
    PR_FatalError ("Cannot open 14.pro", "ex14");
  PR_CenterObject (object_shape, &cenx, &ceny, &cenz);

#if !defined (MSGLIDE) && !defined (WTGLIDE) && !defined (MSGL) && !defined (WTGL)
  /* Set the palette from the textures loaded */
  wsetpalette (0, 255, global_palette);
#endif

  /* Create the main entity */
  entity = PR_CreateEntity (object_shape, "TEST ENTITY");


  /* Initialize the camera */
  camera = PR_AllocCamera ();
  PR_InitializeCamera (camera);
  PR_PositionCameraSource (camera, 0, 0, 400);
  PR_PositionCameraTarget (camera, 0, 0, 0);
  PR_SetCameraMode (camera, CAMFLAG_AIM_TARGET); 
  PR_SetActiveCamera (camera);

 /* Initialize the light */
  PR_AllocLights (&scenelights, 1);
  PR_AllocLights (&userlights, 1);
  PR_SetLightPosition (&userlights, 0, 0, 0, 400);
  PR_SetLightOn (&userlights, 0);
  PR_SetLightType (&userlights, 0, DIRECTIONAL_LIGHT);
  PR_SetLightColor (&userlights, 0, 1.0, 1.0, 1.0);
  PR_SetLightStrength (&userlights, 0, 1.0);
  userlights.NumLights = 1;     /* This sets how many lights are actually used */

  PR_OpenScreen (PR_BACKBUFFER);
  BLACK = PRGFX_MakeColor (0, 0, 0);

  installkbd ();
  animation_count = 0;
  winittimer ();
  wstarttimer (animation_timer, TICKS (60));

  /* main program loop */
  while (!kbdon[KEY_ESC])
    {
     #ifdef WIN32
       UpdateMessages ();
     #endif      

     #if defined (MSDD) || defined (WTDD) || defined (MSGL)  || defined (WTGL) 
       if (PR_LostSurface)
         PR_RestoreAll ();
     #endif

     PR_OpenScreen (PR_BACKBUFFER);

     PRGFX_SetColor (BLACK);
     PRGFX_ClearScreen ();             /* Clear the last frame */

     PR_NewFrame ();                   /* Begin a new frame */

     PR_TransformLights (&userlights);

     move_count = animation_count;
     animation_count = 0;
     UpdateTexture (0);

     PR_RotateEntity (entity, 1.5, 1.2, 0.7);
                                                        /*1.5f * move_count,
                              1.2f * move_count,
                              0.7f * move_count);  */
     PR_TransformEntity (entity);      /* Transform the entity */
     PR_RenderEntity (entity);         /* Render the entity */

     PR_RenderFrame ();                /* Draw the scene */

     PR_Flip (1);                      /* Show the background page */
    }

  /* Free all allocated memory */
  PR_FreeCamera (camera);
  PR_FreeEntity (entity);
  PR_FreeObject (object_shape);
  PR_CloseViewport (&viewport);
  PR_DeleteAllShadeTables ();
  PR_DeleteAllTextures ();
  PR_DeleteAllMaterials ();

  PR_ShutDown ();

  uninstallkbd ();
  wstoptimer ();
  wdonetimer ();

#ifndef WIN32
  wsetmode (3);                        /* Go back to text mode */
#endif
}

