#include <stdlib.h>
#include <math.h>
#include <direct.h>

#include <pr.h>
#include <prgui.h>
#include <prsound.h>

#if !defined (WIN32)
  #include <terrain.h>
  #include <pr3dfx.h>
  #include <glide.h>
#else
  #if defined (MSGLIDE) || defined (WTGLIDE)
    #include <pr3dfx.h>
    #include <glide.h>
  #endif
  #include <terrain.h>
  #include <winutil.h>
#endif

#include "game.h"


PR_OBJECT *EnemyObjects[MAX_ENEMY_SHAPES];
PR_ENTITY *EnemyEntities[MAX_ENEMY_ENTITIES];

#define BIG_COORD 600000.0f


/* Shape numbers */
#define ENEMY_PROBE 0
#define ENEMY_TANK 1

/* User defined data offsets (0-31) */
#define ENEMY_DATA_TYPE 0
#define ENEMY_DATA_FRAME 1
#define ENEMY_DATA_ANIMATED 2
#define ENEMY_DATA_HEALTH 3

#define PROBE_ABOVE 800
#define PROBE_SPEED 20

#define TANK_ABOVE 0
#define TANK_SPEED -10

void LoadEnemyData (void)
{
  EnemyObjects[ENEMY_PROBE] = PR_LoadPRO ("probe.pro", LOAD_NORMAL);
  EnemyObjects[ENEMY_TANK] = PR_LoadPRO ("tank.pro", LOAD_NORMAL);
}



void CheckWeapons (PR_ENTITY *ent, PR_DWORD enemynum)
/* Look for collisions between missiles and objects */
{
PR_DWORD i, j, next;
PR_EMITTER *emit;
PR_POINT dvector, location;

  i = FirstUsedEmitter;
  while (i != -1)
       {
        emit = &EmitterList[i];
        next = emit->Link;

        if (emit->ParticleType == &ParticleTypes[FLARETRAIL_PARTICLE])
          {
           if (PR_SphereInEntity (ent, emit->Location.x, emit->Location.y,
                        emit->Location.z, 125.0f))
             {
              location.x = emit->Location.x;
              location.y = emit->Location.y;
              location.z = emit->Location.z;
              dvector.x = dvector.y = dvector.z = 0.0f;

              ent->userdata[ENEMY_DATA_HEALTH] -= 15;
              if (ent->userdata[ENEMY_DATA_HEALTH] > 0)
                {
                 PR_CreateEmitter (SMOKE_EMITTER, &location, &dvector);
                 PR_CreateEmitter (ASH_EMITTER, &location, &dvector);

                 PRSND_Set3DSoundCoordinate (sound_hitland,
                                             location.x,
                                             location.y,
                                             location.z,
                                             ent);
                 PRSND_Play3DSound (sound_explode1, Waves[WAVE_HITLAND]);
                 PRSND_Update3DSound (sound_hitland);
                 PRSND_UpdateVoices ();
                }
              else
                {
                 PR_SetEntityFlags (ent, FLAG_NORENDER | FLAG_NOTRANSFORM, FLAG_COPY);
                 PR_CreateEmitter (FIRE_EMITTER, &location, &dvector);
                 PR_CreateEmitter (SPARK_EMITTER, &location, &dvector);
                 PR_CreateEmitter (EXPLOSION2_EMITTER, &location, &dvector);

                 for (j = 0; j < 3; j++)
                   {
                    location.x += (rand () % 40) - 20;
                    location.y += (rand () % 20);
                    location.z += (rand () % 40) - 20;

                    dvector.x = (rand () % 40) - 20;
                    dvector.y = (rand () % 20);
                    dvector.z = (rand () % 40) - 20;
                    PR_CreateEmitter (BOUNCEPUFF_EMITTER, &location, &dvector);  
                   }

                 PRSND_Set3DSoundCoordinate (sound_explode1,
                                             location.x,
                                             location.y,
                                             location.z,
                                             NULL);
                 PRSND_Play3DSound (sound_explode1, Waves[WAVE_EXPLODE1]);
                 PRSND_Update3DSound (sound_explode1);

                 PR_PositionEntity (ent, BIG_COORD, BIG_COORD, BIG_COORD);
                 PRSND_Update3DSound (sound_enemy[i]);
                 PRSND_UpdateVoices ();
                }

              PR_RemoveEmitter (emit->Index);
             }
          }

        i = next;
       }
}





void InitializeProbe (PR_ENTITY *ent, PR_DWORD i)
{
PR_REAL x, z, hgt;

  PR_RotateEntity (ent, 0, rand () % 360, 0);
  PR_ScaleEntity (ent, 0.5f, 0.5f, 0.5f);
  ent->shape->bbox.radius *= 0.25f;

  x = (rand () % terrain->width) * WORLD_SCALE;
  z = (rand () % terrain->width) * WORLD_SCALE;

  hgt = PR_GetTerrainHeight (terrain, x, z,
                         terrain->level1_heightpic, terrain->level1_colorpic) + PROBE_ABOVE;

  PR_PositionEntity (ent, x, hgt + rand () % 1000, z);

  ent->userdata[ENEMY_DATA_ANIMATED] = 1;
  ent->userdata[ENEMY_DATA_HEALTH] = 100;

  sound_enemy[i] = PRSND_Alloc3DSound ();
  PRSND_Play3DSound (sound_enemy[i], Waves[WAVE_PROBE]);
  PRSND_Set3DSoundVolume (sound_enemy[i], 64);
  PRSND_Set3DSoundVolumeMode (sound_enemy[i], SOUND_3D_VOLUME);
  PRSND_Set3DSoundVolumeFactor (sound_enemy[i], 5000);
  PRSND_Set3DSoundPanning (sound_enemy[i], 128);
  PRSND_Set3DSoundPanningMode (sound_enemy[i], SOUND_3D_PANNING);
  PRSND_Set3DSoundPanningFactor (sound_enemy[i], 5000);
  PRSND_Set3DSoundShiftMode (sound_enemy[i], SOUND_DOPPLER_ON);
  PRSND_Set3DSoundShiftFactor (sound_enemy[i], 0.0005f);
  PRSND_Set3DSoundShiftSlide (sound_enemy[i], 100);
  PRSND_Set3DSoundCoordinate (sound_enemy[i], 0, 0, 0, ent);
  PRSND_Update3DSound (sound_enemy[i]);
}



void InitializeTank (PR_ENTITY *ent, PR_DWORD i)
{
PR_REAL x, z;

  PR_RotateEntity (ent, 0, rand () % 360, 0);
  PR_ScaleEntity (ent, 0.5f, 0.5f, 0.5f);
  ent->shape->bbox.radius *= 0.99f;

  x = (rand () % terrain->width) * WORLD_SCALE;
  z = (rand () % terrain->width) * WORLD_SCALE;

  PR_PositionEntity (ent, x, 0, z);

  ent->userdata[ENEMY_DATA_ANIMATED] = 0;
  ent->userdata[ENEMY_DATA_HEALTH] = 200;

  sound_enemy[i] = PRSND_Alloc3DSound ();
  PRSND_Play3DSound (sound_enemy[i], Waves[WAVE_TANK]);
  PRSND_Set3DSoundVolume (sound_enemy[i], 64);
  PRSND_Set3DSoundVolumeMode (sound_enemy[i], SOUND_3D_VOLUME);
  PRSND_Set3DSoundVolumeFactor (sound_enemy[i], 5000);
  PRSND_Set3DSoundPanning (sound_enemy[i], 128);
  PRSND_Set3DSoundPanningMode (sound_enemy[i], SOUND_3D_PANNING);
  PRSND_Set3DSoundPanningFactor (sound_enemy[i], 5000);
  PRSND_Set3DSoundShiftMode (sound_enemy[i], SOUND_DOPPLER_ON);
  PRSND_Set3DSoundShiftFactor (sound_enemy[i], 0.0005f);
  PRSND_Set3DSoundShiftSlide (sound_enemy[i], 100);
  PRSND_Set3DSoundCoordinate (sound_enemy[i], 0, 0, 0, ent);
  PRSND_Update3DSound (sound_enemy[i]);
}


void UpdateProbeNear (PR_ENTITY *ent, PR_DWORD ticks_passed)
{
PR_REAL hgt;
PR_ORIENTATION *orient;

  PR_RotateEntity (ent, 0, (rand () % 10) - 5, 0);

  /* Get a pointer to the orientation */
  orient = &ent->orientation;

  /* Prevent the entity from going below the landscape */
  hgt = PR_GetTerrainHeight (terrain, orient->location.x, orient->location.z,
                             terrain->level1_heightpic, terrain->level1_colorpic) + PROBE_ABOVE;
  if (orient->location.y < hgt)
    orient->location.y = hgt;
}


void UpdateProbeFar (PR_ENTITY *ent, PR_DWORD ticks_passed)
{
PR_ORIENTATION *orient;

  /* Get a pointer to the orientation */
  orient = &ent->orientation;

  /* Set up a vector in the Z direction */
  tvector[0] = 0;
  tvector[1] = (rand () % 10) - 5;
  tvector[2] = PROBE_SPEED;
  PR_Transform (orient->rot_mat);
  /* Transform it through the rotation matrix to get the movement vector */

  PR_MoveEntity (ent, tvector[0], tvector[1], tvector[2]);
}




void GenerateGroundEnemyRotation (PR_ENTITY *ent)
/* Written by Phillip Schuster (www.voodoospotlight.com) */
{
PR_POINT p[4];
PR_POINT p1[4];
PR_POINT e;
PR_ORIENTATION *orient;
PR_DWORD i;
PR_REAL length[4];
PR_REAL angle, angle2;
PR_REAL dist1, dist2, dist, theta;
PR_POINT rot;
PR_POINT u,v;

  PR_RotateEntityAbs (ent,0,0,0);
  e=ent->orientation.location;
  orient=&ent->orientation;

  tvector[0] = 70;
  tvector[1] = 0;
  tvector[2] = 100;
  PR_Transform (orient->rot_mat);
  p[0].x = tvector[0] + e.x;
  p[0].y = tvector[1] + e.y;
  p[0].z = tvector[2] + e.z;

  tvector[0] = -70;
  tvector[1] = 0;
  tvector[2] = 100;
  PR_Transform (orient->rot_mat);
  p[1].x = tvector[0] + e.x;
  p[1].y = tvector[1] + e.y;
  p[1].z = tvector[2] + e.z;

  tvector[0] = -70;
  tvector[1] = 0;
  tvector[2] = -95;
  PR_Transform (orient->rot_mat);
  p[2].x = tvector[0] + e.x;
  p[2].y = tvector[1] + e.y;
  p[2].z = tvector[2] + e.z;

  tvector[0] = 70;
  tvector[1] = 0;
  tvector[2] = -95;
  PR_Transform (orient->rot_mat);
  p[3].x = tvector[0] + e.x;
  p[3].y = tvector[1] + e.y;
  p[3].z = tvector[2] + e.z;


  for (i = 0; i <= 3; i++)
    {
     p1[i].x = p[i].x;
     p1[i].z = p[i].z;
     p1[i].y = PR_GetTerrainHeight (terrain, p1[i].x, p1[i].z, terrain->level1_heightpic,terrain->level1_colorpic);
    }

  for (i = 0; i <= 3; i++)
    length[i] = p[i].y - p1[i].y;

  u.x = p1[0].x - p1[1].x;
  u.y = p1[0].y - p1[1].y;
  u.z = p1[0].z - p1[1].z;

  v.x = p1[0].x - p1[1].x;
  v.y = 0;
  v.z = p1[0].z - p1[1].z;

  theta = (u.x*v.x)+(u.y*v.y)+(u.z*v.z);
  dist1 = sqrt((u.x*u.x)+(u.y*u.y)+(u.z*u.z));
  dist2 = sqrt((v.x*v.x)+(v.y*v.y)+(v.z*v.z));
  dist = dist1 * dist2;

  if (length[0] >= length[1])
   angle = -(acos(theta/dist))*(180 / PR_PI);
  else
   angle = (acos(theta/dist))*(180 / PR_PI);


  u.x = p1[3].x - p1[0].x;
  u.y = p1[3].y - p1[0].y;
  u.z = p1[3].z - p1[0].z;

  v.x = p1[3].x - p1[0].x;
  v.y = 0;
  v.z = p1[3].z - p1[0].z;

  theta = (u.x*v.x)+(u.y*v.y)+(u.z*v.z);
  dist1 =sqrt((u.x*u.x)+(u.y*u.y)+(u.z*u.z));
  dist2 =sqrt((v.x*v.x)+(v.y*v.y)+(v.z*v.z));
  dist = dist1*dist2;

  if (length[3]>=length[0])
    angle2 = -(acos(theta/dist))*(180 / PR_PI);
  else                                    
    angle2 = (acos(theta/dist))*(180 / PR_PI);

  rot = ent->orientation.rot;

  PR_RotateEntityAbs (ent,angle2,orient->rot.y,angle);
}



void UpdateTankNear (PR_ENTITY *ent, PR_DWORD ticks_passed)
{
PR_REAL hgt;
PR_ORIENTATION *orient;

  PR_RotateEntity (ent, 0, (rand () % 10) - 5, 0);

  /* Get a pointer to the orientation */
  orient = &ent->orientation;

  /* Prevent the entity from going below the landscape */
  hgt = PR_GetTerrainHeight (terrain, orient->location.x, orient->location.z,
                         terrain->level1_heightpic, terrain->level1_colorpic) + TANK_ABOVE;
  orient->location.y = hgt;
  GenerateGroundEnemyRotation (ent);
}


void UpdateTankFar (PR_ENTITY *ent, PR_DWORD ticks_passed)
{
PR_ORIENTATION *orient;

  /* Get a pointer to the orientation */
  orient = &ent->orientation;

  /* Set up a vector in the Z direction */
  tvector[0] = 0;
  tvector[1] = 0;
  tvector[2] = TANK_SPEED;
  PR_Transform (orient->rot_mat);
  /* Transform it through the rotation matrix to get the movement vector */

  PR_MoveEntity (ent, tvector[0], tvector[1], tvector[2]);
}



void InitializeEnemies (void)
{
PR_DWORD i;
PR_DWORD type;

  for (i = 0; i < MAX_ENEMY_ENTITIES; i++)
    {
     if (i < MAX_ENEMY_ENTITIES / 2)
       type = ENEMY_PROBE;
     else
       type = ENEMY_TANK;

     EnemyEntities[i] = PR_CreateEntity (EnemyObjects[type], "enemy");
     EnemyEntities[i]->userdata[ENEMY_DATA_TYPE] = type;
     EnemyEntities[i]->userdata[ENEMY_DATA_FRAME] = 0;

     switch (type)
       {
        case ENEMY_PROBE: InitializeProbe (EnemyEntities[i], i);
        break;
        case ENEMY_TANK: InitializeTank (EnemyEntities[i], i);
        break;
       }
    }
}


void UpdateEnemies (PR_DWORD ticks_passed)
{
PR_DWORD i;
PR_DWORD dx, dz;
PR_REAL tsize;
/*
  PRGFX_SetColor (PRGFX_MakeColor (32, 32, 32));
  PR_SetConstantZ (1);
  PRGFX_Rectangle (566, 10, 630, 74);
*/
  tsize = WORLD_SCALE * terrain_size;

  for (i = 0; i < MAX_ENEMY_ENTITIES; i++)
   if (!(PR_GetEntityFlags (EnemyEntities[i]) & FLAG_NOTRANSFORM))
     {
      /* Entity is still active */

      /* Update the movement and AI */
      switch (EnemyEntities[i]->userdata[ENEMY_DATA_TYPE])
        {
         case ENEMY_PROBE:
           UpdateProbeFar (EnemyEntities[i], ticks_passed);
           break;
         case ENEMY_TANK:
           UpdateTankFar (EnemyEntities[i], ticks_passed);
           break;
        }


      /* Now see if the enemy is within viewing distance */
      EntityCoordinateFix (EnemyEntities[i]);
      dx = EnemyEntities[i]->orientation.location.x - camera->source.x;
      dz = EnemyEntities[i]->orientation.location.z - camera->source.z;

      if ((dx > -tsize) &&
          (dx < tsize) &&
          (dz > -tsize) &&
          (dz < tsize))
        {
         /* It is within range. */

         /* Update the movement and AI */
         switch (EnemyEntities[i]->userdata[ENEMY_DATA_TYPE])
           {
            case ENEMY_PROBE:
              UpdateProbeNear (EnemyEntities[i], ticks_passed);
              break;
            case ENEMY_TANK:
              UpdateTankNear (EnemyEntities[i], ticks_passed);
              break;
           }

         if (EnemyEntities[i]->userdata[ENEMY_DATA_ANIMATED])
           {
            /* Advance and loop the animation */
            EnemyEntities[i]->userdata[ENEMY_DATA_FRAME] =
              (EnemyEntities[i]->userdata[ENEMY_DATA_FRAME] + (ticks_passed<<2)) %
               EnemyEntities[i]->shape->num_frames;

            PR_AnimateEntity (EnemyEntities[i], EnemyEntities[i]->userdata[ENEMY_DATA_FRAME]);
           }

         PRSND_Update3DSound (sound_enemy[i]);

         CheckWeapons (EnemyEntities[i], i);

         PR_TransformEntity (EnemyEntities[i]);
         PR_RenderEntity (EnemyEntities[i]);

         /* Draw the radar blips */
/*         PRGFX_SetColor (PRGFX_MakeColor (63, 0, 0));
         dx = 598 + EnemyEntities[i]->shape->segment_list->bbox.tbox[0].x * 0.003125f;
         dz = 42  - EnemyEntities[i]->shape->segment_list->bbox.tbox[0].z * 0.003125f;
         PRGFX_Bar (dx, dz, dx, dz);*/
        }
     }
}


