#include <pr.h>
#include <prsound.h>

#include <terrain.h>

#include "game.h"

/* Sound Effects */
PR_DWORD Waves[MAXWAVE];

PR_3DSOUND *sound_weap1;
PR_3DSOUND *sound_hitland;
PR_3DSOUND *sound_explode1;
PR_3DSOUND *sound_scrape;
PR_3DSOUND *sound_enemy[MAX_ENEMY_ENTITIES];


void LoadSoundSet (void)
{
  Waves[WAVE_WEAPON1]  = PRSND_LoadSound (".\\sounds\\weap1.wav");
  Waves[WAVE_HITLAND]  = PRSND_LoadSound (".\\sounds\\hitland.wav");
  Waves[WAVE_EXPLODE1] = PRSND_LoadSound (".\\sounds\\explode1.wav");
  Waves[WAVE_SCRAPE]   = PRSND_LoadSound (".\\sounds\\scrape.wav");
  Waves[WAVE_PROBE]    = PRSND_LoadSound (".\\sounds\\probe.wav");
  Waves[WAVE_TANK]     = PRSND_LoadSound (".\\sounds\\tank.wav");

  PRSND_SetLoopStart (Waves[WAVE_PROBE], 0);
  PRSND_SetLoopEnd (Waves[WAVE_PROBE], PRSND_GetLength (Waves[WAVE_PROBE]));
  PRSND_SetLoopMode (Waves[WAVE_PROBE], TRUE);

  PRSND_SetLoopStart (Waves[WAVE_TANK], 0);
  PRSND_SetLoopEnd (Waves[WAVE_TANK], PRSND_GetLength (Waves[WAVE_TANK]));
  PRSND_SetLoopMode (Waves[WAVE_TANK], TRUE);

  sound_weap1 = PRSND_Alloc3DSound ();
  PRSND_Set3DSoundVolume (sound_weap1, 64);
  PRSND_Set3DSoundVolumeMode (sound_weap1, SOUND_3D_VOLUME);
  PRSND_Set3DSoundVolumeFactor (sound_weap1, 2000);
  PRSND_Set3DSoundPanning (sound_weap1, 128);
  PRSND_Set3DSoundPanningMode (sound_weap1, SOUND_3D_PANNING);
  PRSND_Set3DSoundPanningFactor (sound_weap1, 2000);
  PRSND_Set3DSoundShiftMode (sound_weap1, SOUND_DOPPLER_OFF);
  PRSND_Set3DSoundShiftFactor (sound_weap1, 0.5);
  PRSND_Set3DSoundCoordinate (sound_weap1, 0, 0, 0, NULL);

  sound_hitland = PRSND_Alloc3DSound ();
  PRSND_Set3DSoundVolume (sound_hitland, 64);
  PRSND_Set3DSoundVolumeMode (sound_hitland, SOUND_3D_VOLUME);
  PRSND_Set3DSoundVolumeFactor (sound_hitland, 9000);
  PRSND_Set3DSoundPanning (sound_hitland, 128);
  PRSND_Set3DSoundPanningMode (sound_hitland, SOUND_3D_PANNING);
  PRSND_Set3DSoundPanningFactor (sound_hitland, 5000);
  PRSND_Set3DSoundShiftMode (sound_hitland, SOUND_DOPPLER_OFF);
  PRSND_Set3DSoundShiftFactor (sound_hitland, 0.005f);
  PRSND_Set3DSoundCoordinate (sound_hitland, 0, 0, 0, NULL);

  sound_explode1 = PRSND_Alloc3DSound ();
  PRSND_Set3DSoundVolume (sound_explode1, 64);
  PRSND_Set3DSoundVolumeMode (sound_explode1, SOUND_3D_VOLUME);
  PRSND_Set3DSoundVolumeFactor (sound_explode1, 5000);
  PRSND_Set3DSoundPanning (sound_explode1, 128);
  PRSND_Set3DSoundPanningMode (sound_explode1, SOUND_3D_PANNING);
  PRSND_Set3DSoundPanningFactor (sound_explode1, 2000);
  PRSND_Set3DSoundShiftMode (sound_explode1, SOUND_DOPPLER_OFF);
  PRSND_Set3DSoundShiftFactor (sound_explode1, 0.005f);
  PRSND_Set3DSoundCoordinate (sound_explode1, 0, 0, 0, NULL);

  sound_scrape = PRSND_Alloc3DSound ();
  PRSND_Set3DSoundVolume (sound_scrape, 64);
  PRSND_Set3DSoundVolumeMode (sound_scrape, SOUND_3D_VOLUME);
  PRSND_Set3DSoundVolumeFactor (sound_scrape, 4000);
  PRSND_Set3DSoundPanning (sound_scrape, 128);
  PRSND_Set3DSoundPanningMode (sound_scrape, SOUND_3D_PANNING);
  PRSND_Set3DSoundPanningFactor (sound_scrape, 4000);
  PRSND_Set3DSoundShiftMode (sound_scrape, SOUND_DOPPLER_OFF);
  PRSND_Set3DSoundShiftFactor (sound_scrape, 0.005f);
  PRSND_Set3DSoundCoordinate (sound_scrape, 0, 0, 0, NULL);
}


/* ---------------------------------------------------------------------
   Initialize the sound system
   --------------------------------------------------------------------- */
void InitializeAudio (void)
{
  /* initialize audio library */
  PRSND_Initialize ();

  PRSND_AllocSounds (10);

  /* open audio device */
  PRSND_OpenAudio (AUDIO_DEVICE_MAPPER,
                   AUDIO_FORMAT_16BITS | AUDIO_FORMAT_STEREO,
                   22050);

  PRSND_LoadSong ("space.s3m");

  PRSND_InitializeVoices (16);

  LoadSoundSet ();

  PRSND_PlaySong ();
  PRSND_SetSongVolume (32);

  PRSND_SetOrigin (0, 0, 0);
}


void CloseAudio (void)
{
  PRSND_CloseVoices ();
  PRSND_DeleteAllSounds ();
  PRSND_CloseAudio ();
}

