#include <math.h>
#include <malloc.h>
#include <stdlib.h>
#include <direct.h>
#include <conio.h>
#include <string.h>

#include <pr.h>
#include <prgui.h>
#include <prsound.h>

#if defined (MSGLIDE) || defined (WTGLIDE)
#include <pr3dfx.h>
#include <glide.h>
#include <terrain.h>
#include <winutil.h>

#elif defined (WIN32)

#include <terrain.h>
#include <winutil.h>
#else
#include <terrain.h>

 #ifdef __3DFX__
  #include <pr3dfx.h>
  #include <glide.h>
 #endif
#endif

#include <prdev.h>

#include "setup.h"
#include "game.h"


/* 3D Engine */
PR_CAMERA *   camera;
PR_VIEWPORT   viewport;                  /* Our viewport structure */
PR_DWORD      vwidth, vheight;           /* Viewport width and height */
PR_DWORD      camera_view = 0;
PR_REAL       camera_fov = 1.5f;
PR_REAL viewx = 0, viewy = 0, viewz = 0;


/* 3D Objects */
PR_OBJECT *sky_shape;
PR_ENTITY *sky_entity;
PR_OBJECT *spaceship_shape;
PR_ENTITY *spaceship_entity;


/* Terrain */
PR_TERRAIN *terrain;
PR_DWORD terrain_size;


/* Textures and bitmaps */
block tilesprites[1024];


/* Palettes and Colors */
color temppal[256];

/* Lights */
PR_LIGHTLIST userlights;
PR_LIGHTLIST explosionlights;
PR_LIGHTLIST sunlight;
PR_LIGHTLIST weaponlight;

/* User Input */
PR_REAL spinspeed;
PR_MATRIX rollmatrix;
PR_REAL TILT_SPEED;
PR_REAL MAX_TILT_SPEED;
PR_REAL TILT_DRAG;

#define RELOAD_TIME 10
PR_DWORD shoot_reload = 0;
PR_DWORD shoot_wing = 0;

/* Frame Rate Control  */
PR_DWORD animation_count = 0;   /* ticks since last draw */
PR_DWORD ticks = 0;             /* Total number of ticks passed for fps */
PR_DWORD updates = 0;           /* Total number of updates */
PR_DWORD framerate;             /* Resulting frame rate (updates/ticks/tickrate) */



void EntityCoordinateFix (PR_ENTITY *ent);


typedef struct {
  char idlength;
  char colormaptype;
  char imagetype;
  unsigned short colormapstart;
  unsigned short colormaplength;
  char colormapbits;
  unsigned short xstart, ystart, width, depth;
  char bits;
  char descriptor;
} PR_TGAHEADER;
PR_TGAHEADER tga;


void DumpScreenshot (void)
{
block pic;
FILE *out;
unsigned short *temp;
PR_UWORD rgb;
PR_DWORD l;
PR_DWORD r, g, b;
char outfile[80];
PR_DWORD number;


  number = 1;
  for (;;)
   {
    sprintf (outfile, "screen%i.tga", number);
    out = fopen (outfile, "rb");
    if (out == NULL)
      break;
    number++;
    fclose (out);
   }

  pic = PRGFX_NewBlock (0, 0, 639, 479);
  temp = (unsigned short *)pic;

  memset ((char *)&tga, 0, sizeof (PR_TGAHEADER));
  tga.descriptor = 0x20;
  tga.width = 640;
  tga.depth = 480;
  tga.idlength = 0;
  tga.imagetype = 2;
  tga.bits = 24;

  sprintf (outfile, "screen%i.tga", number);
  out = fopen (outfile, "wb");
  fwrite (&tga, sizeof (PR_TGAHEADER), 1, out);

  for (l = 0; l < 640*480; l++)
    {
     rgb = *temp++;

     r = (rgb >> 11);
     g = (rgb - (r << 11)) >> 5;
     b = (rgb - (r << 11) - (g << 5));

     r = (PR_REAL)r * 1.5f;
     g = (PR_REAL)g * 1.5f;
     b = (PR_REAL)b * 1.5f;
     if (r > 31)
       r = 31;
     if (g > 63)
       g = 63;
     if (b > 31)
       b = 31;

     fputc (b << 3, out);
     fputc (g << 2, out);
     fputc (r << 3, out);
    }

  fclose (out);

  free (pic);
}



void EntityCoordinateFix (PR_ENTITY *ent)
{
PR_DWORD dx, dz;
PR_DWORD width, halfwidth;

  dx = ent->orientation.location.x - camera->source.x;
  dz = ent->orientation.location.z - camera->source.z;

  width = terrain->width * terrain->world_scale;
  halfwidth = width >> 1;

  if (dx > halfwidth)
    ent->orientation.location.x -= width;
  if (dx < -halfwidth)
    ent->orientation.location.x += width;

  if (dz > halfwidth)
    ent->orientation.location.z -= width;
  if (dz < -halfwidth)
    ent->orientation.location.z += width;
}



void ThrustSpaceship (PR_REAL speed)
/* Move the spaceship forward or backward (negative speed) */
{
PR_ORIENTATION *orient;


  /* Get a pointer to the ship's orientation */
  orient = &spaceship_entity->orientation;

  /* Set up a vector in the Z direction */
  tvector[0] = 0;
  tvector[1] = 0;
  tvector[2] = speed;
  PR_Transform (orient->rot_mat);
  /* Transform it through the ship's rotation matrix to get the
     movement vector */

  PR_MoveEntity (spaceship_entity,
     tvector[0], tvector[1], tvector[2]);

  EntityCoordinateFix (spaceship_entity);
}



void ChooseCameraView (void)
{
  switch (camera_view)
    {
     case 0:
        tvector[0] = 0;         /* Cockpit view */
        tvector[1] = 1;
        tvector[2] = -10;
        break;
     case 1:
        tvector[0] = 0;         /* Rear view */
        tvector[1] = 1;
        tvector[2] = 20;
        break;
     case 2:                    /* External view */
        tvector[0] = -yawspeed * 15;
        tvector[1] = -pitchspeed * 10 + 70;
        tvector[2] = CAMERA_DISTANCE; 
        break;
     case 3:                    /* Free camera */
        tvector[0] = viewx;
        tvector[1] = viewy;
        tvector[2] = CAMERA_DISTANCE + viewz;
        break;
    }
}



void ExternalCamera (void)
/* Make the camera stay behind the spaceship */
{
PR_ORIENTATION *orient;
PR_REAL x, y, z, hgt;

  /* Get a pointer to the ship's orientation */
  orient = &spaceship_entity->orientation;

  /* Set up a vector behind the ship somewhere */
  ChooseCameraView ();

  PR_Transform (orient->rot_mat);
  /* Transform it through the ship's rotation matrix to get the
     movement vector */

  x = spaceship_entity->orientation.location.x;
  y = spaceship_entity->orientation.location.y;
  z = spaceship_entity->orientation.location.z;
  /* Set the target to be the middle of the ship */

  x += tvector[0];      /* Add the vector to move it behind the ship */
  y += tvector[1];
  z += tvector[2];

  /* Now we can't let the camera below the ground, so check the
     height */
  hgt = PR_GetTerrainHeight (terrain, x,
                                      z,
                                      terrain->level1_heightpic,
                                      terrain->level1_colorpic)
                                      + ABOVE_GROUND;
  if (y < hgt)
    y = hgt;

  PR_PositionCameraSource (camera, x, y, z);

//  PR_SetLightPosition (&userlights, 0, x, y, z);

  /* Set up a vector in the Y direction */
  tvector[0] = 0;
  tvector[1] = 1;
  tvector[2] = 0;
  if (camera_view != 3)
    PR_Transform (orient->rot_mat);

  camera->up.x = tvector[0];            /* Set the up vector of the camera */
  camera->up.y = tvector[1];            
  camera->up.z = tvector[2];            
}




/* -----------------------------------------------------------------------
   Wave code
   ----------------------------------------------------------------------- */

void PR_TerrainWaveFunction (PR_WORD *col, PR_WORD *hgt, PR_WORD *shd,
                             PR_DWORD x, PR_DWORD y)
{
 /* Dummy callback routine requried for PR terrain engine */
  return;
}



PR_DWORD PR_HitLand (PR_TERRAIN *terrain, PR_REAL x, PR_REAL y, PR_REAL z)
{
PR_REAL hgt1;

 hgt1 = PR_GetTerrainHeight (terrain, x, z, terrain->level1_heightpic, terrain->level1_colorpic);

 if (y < hgt1)
   return 1;
 else
   return 0;
}







/* -----------------------------------------------------------------------
   Missile code
   ----------------------------------------------------------------------- */
void ModifyLand (PR_REAL x, PR_REAL y, PR_REAL z)
{
PR_DWORD offset;
PR_DWORD wx, wy;
unsigned char *bits;
PR_DWORD pitch;

  wx = z / WORLD_SCALE;
  wy = x / WORLD_SCALE;
  while (wx < 0)
    wx += terrain->width;
  while (wx >= terrain->width)
    wx -= terrain->width;
  while (wy < 0)
    wy += terrain->height;
  while (wy >= terrain->height)
    wy -= terrain->height;

#if defined (MSDD) || defined (WTDD)
  bits = (unsigned char *)wgetblocksurface (terrain->level1_heightpic, &pitch);
  offset = wy * pitch + wx;
#else
  bits = terrain->level1_heightpic;
  offset = wy * terrain->width + wx + 4;
#endif

  if (bits == NULL)
     return;
  
  bits += offset;
  if (*bits > 5)
    *bits -= 5;

#if defined (MSDD) || defined (WTDD)
  bits = wgetblocksurface (terrain->level1_shadepic, &pitch);
  offset = wy * pitch + wx;
#else
  bits = terrain->level1_shadepic;
#endif

 
  if (bits == NULL)
     return;

  bits += offset;
  if (*bits > 8)
    *bits -= 8;
  else
    *bits = 0;

  PR_TerrainModify (terrain);

#if defined (MSDD) || defined (WTDD)
  wunlocksurface (terrain->level1_shadepic);
  wunlocksurface (terrain->level1_heightpic);
#endif
}



void AddExplosion (PR_REAL x, PR_REAL y, PR_REAL z)
{
PR_DWORD i;

  ModifyLand (x,y,z);

  for (i = 0; i < 20; i++)
    if (!PR_GetLightState (&explosionlights, i))
      {
       PR_SetLightOn (&explosionlights, i);
       PR_SetLightColor (&explosionlights, i, 1.0, 0.7, 0);
       PR_SetLightStrength (&explosionlights, i, 1.0);
       PR_SetLightPosition (&explosionlights, i, x, y, z);
       break;
      }
}


void UpdateExplosions (void)
{
PR_DWORD i;
PR_REAL strength;

  for (i = 0; i < 20; i++)
    if (PR_GetLightState (&explosionlights, i))
      {
       strength = PR_GetLightStrength (&explosionlights, i) - 0.025;

       if (strength < 0)
         PR_SetLightOff (&explosionlights, i);
       else
         PR_SetLightStrength (&explosionlights, i, strength);
      }
}



/* ---------------------------------------------------------------------- */
/* Makes sure the camera is above the landscape for the first frame */
/* ---------------------------------------------------------------------- */
void SetInitialPosition (PR_ENTITY *ent)
{
PR_REAL hgt;

  hgt = PR_GetTerrainHeight (terrain, ent->orientation.location.x,
                                      ent->orientation.location.z,
                                      terrain->level1_heightpic,
                                      terrain->level1_colorpic)
                                      + ABOVE_GROUND;


  PR_PositionCameraSource (camera,
                      ent->orientation.location.x,
                      hgt + ABOVE_GROUND,
                      ent->orientation.location.z);
}





/* ---------------------------------------------------------------------- */
/* Checks for ground collision in the flying model */
/* ---------------------------------------------------------------------- */
void SetFlyingHeight (PR_ENTITY *ent)
{
PR_REAL hgt1;
PR_POINT dvector, location;

  hgt1 = PR_GetTerrainHeight (terrain, ent->orientation.location.x,
                                       ent->orientation.location.z,
                                       terrain->level1_heightpic,
                                       terrain->level1_colorpic)
                                       + ABOVE_GROUND;

  if (ent->orientation.location.y < hgt1)
    {
     ent->orientation.location.y = hgt1;

     if (!PRSND_Get3DSoundStatus (sound_scrape))
       {
        location.x = spaceship_entity->orientation.location.x;
        location.y = spaceship_entity->orientation.location.y;
        location.z = spaceship_entity->orientation.location.z;
        dvector.x = dvector.y = dvector.z = 0.0f;

        PRSND_Set3DSoundCoordinate (sound_scrape,
                                    location.x,
                                    location.y,
                                    location.z,
                                    spaceship_entity);
        PRSND_Play3DSound (sound_scrape, Waves[WAVE_SCRAPE]);
        PRSND_Update3DSound (sound_scrape);
        PRSND_UpdateVoices ();
        PR_CreateEmitter (ASH_EMITTER, &location, &dvector);
       }
    }

  if (ent->orientation.location.y > MAX_HEIGHT)
     ent->orientation.location.y = MAX_HEIGHT;
}






PR_DWORD tilemap[256*6];
/* ---------------------------------------------------------------------- */
/* Creates a material list with mip mapping for the landscape */
/* ---------------------------------------------------------------------- */
void InitializeMaterials (void)
{
PR_MATERIAL *m;
PR_DWORD i;
char tilename[80];

  /* Create a NULL material for objects */
  m = &PR_ObjectMaterialList[0];
  PR_SetMaterialName (m, "NULL");
  PR_SetMaterialMethod (m, NULL_TYPE);


  /* Now load the tiles from a sprite file, and set up 4 levels of
     mip mapping for each.  */

  wloadsprites (global_palette, "landtile.spr", tilesprites, 0, 1023);
  /* Load the images into the tilesprites array */
#if !defined (MSGLIDE) && !defined (WTGLIDE) && !defined (MSGL) && !defined (WTGL)
  if ((device == DEVICE_VGA) || (device == DEVICE_SVGA))
    wsetpalette (0, 255, global_palette);
#endif

  /* Set the null type for land tiles
     This is to allow for holes in the landscape. */
//  m = &PR_ObjectMaterialList[OBJ_TILES];
//  PR_SetMaterialName (m, "NULL");
//  PR_SetMaterialMethod (m, NULL_TYPE);


  /* For each tile image, make 4 materials for the mip mapping */

  for (i = 1; i <= NUM_TILES; i++)
    {
     m = &PR_ObjectMaterialList[OBJ_TILES + i];

     /* We already have the images loaded in tilesprites, so just
        make a pointer to them */
     sprintf (tilename, "tile%i", i-1);
     if (tilesprites[i] != NULL)
       tilemap[i] = PR_AddTexture (tilename, tilesprites[i]);

     m = &PR_ObjectMaterialList[OBJ_TILES + i];
     PR_SetMaterialName (m, "tile1");

     PR_SetMaterialMethod (m, T_FOG_GOURAUD_TEXTURED);

     PR_SetMaterialBaseColor (m, 0);
     PR_SetMaterialColor (m, 255, 255, 255);
     PR_SetMaterialTexture (m, tilemap[i]);
     PR_SetMaterialTable (m, 0);
     PR_SetMaterialShades (m, 31);
     PR_SetMaterialEnvironmentMap (m, 0);
    }
}



void InitializeTexture (void)
{
  PR_Settings.LoadPalette = 1;
  InitializeMaterials ();
}



/* ---------------------------------------------------------------------- */
/* Check the mouse and keyboard input */
/* ---------------------------------------------------------------------- */
void UserInput (void)
{
  if (kbdon[KEY_S])
    DumpScreenshot ();

  if ((kbdon[KEY_LBRACE]) && (camera_fov > 0.1))
    {
     camera_fov -= 0.025f;
     camera->fov = camera_fov / 3.0f * PR_PI;
    }
  if ((kbdon[KEY_RBRACE]) && (camera_fov < 2.5))
    {
     camera_fov += 0.025f;
     camera->fov = camera_fov / 3.0f * PR_PI;
    }


  if (kbdon[KEY_F1])
    camera_view = 0;
  if (kbdon[KEY_F2])
    camera_view = 1;
  if (kbdon[KEY_F3])
    camera_view = 2;
  if (kbdon[KEY_F4])
    camera_view = 3;

  if (camera_view == 3)
    {
     if (kbdon[KEY_INSERT])
       viewx += 10;
     if (kbdon[KEY_DEL])
       viewx -= 10;
     if (kbdon[KEY_HOME])
       viewy += 10;
     if (kbdon[KEY_END])
       viewy -= 10;
     if (kbdon[KEY_PGUP])
       viewz += 10;
     if (kbdon[KEY_PGDN])
       viewz -= 10;
    }

  if (kbdon[KEY_Q])
    SmoothMove (&spinspeed, TILT_SPEED, MAX_TILT_SPEED, TILT_DRAG);
  else if (kbdon[KEY_W])
    SmoothMove (&spinspeed, -TILT_SPEED, MAX_TILT_SPEED, TILT_DRAG);
  else
    SmoothMove (&spinspeed, 0, MAX_TILT_SPEED, TILT_DRAG);
 
  if (kbdon[KEY_UP])
    SmoothMove (&pitchspeed, PITCH_SPEED, MAX_PITCH_SPEED, PITCH_DRAG);
  else if (kbdon[KEY_DOWN])
    SmoothMove (&pitchspeed, -PITCH_SPEED, MAX_PITCH_SPEED, PITCH_DRAG);
  else
    SmoothMove (&pitchspeed, 0, MAX_PITCH_SPEED, PITCH_DRAG);
 
  if (kbdon[KEY_LEFT])
    {
     SmoothMove (&yawspeed, -YAW_SPEED, MAX_YAW_SPEED, YAW_DRAG);
     SmoothMove (&rollspeed, ROLL_SPEED, MAX_ROLL_SPEED, ROLL_DRAG);
    }
  else if (kbdon[KEY_RIGHT])
    {
     SmoothMove (&yawspeed, YAW_SPEED, MAX_YAW_SPEED, YAW_DRAG);
     SmoothMove (&rollspeed, -ROLL_SPEED, MAX_ROLL_SPEED, ROLL_DRAG);
    }
  else
    {
     SmoothMove (&yawspeed, 0, MAX_YAW_SPEED, YAW_DRAG);
     SmoothMove (&rollspeed, 0, MAX_ROLL_SPEED, ROLL_DRAG);
    }

  memcpy (spaceship_entity->orientation.rot_mat, rollmatrix, sizeof (PR_MATRIX));
  PR_RotateEntity (spaceship_entity, pitchspeed, yawspeed, spinspeed);
  memcpy (rollmatrix, spaceship_entity->orientation.rot_mat, sizeof (PR_MATRIX));
  PR_RotateEntity (spaceship_entity, 0, 0, rollspeed);

  if (kbdon[KEY_A]) /* forward */
    SmoothMove (&currentspeed, TRANSLATE_SPEED, MAX_TRANSLATE_SPEED, TRANSLATE_DRAG);
  else if (kbdon[KEY_Z]) /* backward */
    SmoothMove (&currentspeed, -TRANSLATE_SPEED, MAX_TRANSLATE_SPEED, TRANSLATE_DRAG);
  else
    SmoothMove (&currentspeed, 0, MAX_TRANSLATE_SPEED, TRANSLATE_DRAG);

  if ((kbdon[KEY_CTRL]) && (shoot_reload < 1))
    {
     FireWeapon1 (spaceship_entity);
     shoot_reload = RELOAD_TIME;
    }

  ThrustSpaceship (currentspeed + 20);
        
  SetFlyingHeight (spaceship_entity);

  if (camera_view != 3)
    camera->tilt_angle = rollspeed / 2;
  else
    camera->tilt_angle = 0;

  /* Tilt angle is based on the amount you are moving left or right */

  /* Now that the ship has moved, figure out where the camera should be */
  ExternalCamera ();

  /* Wrap camera around if needed */

  if (camera->source.x < 0)
    camera->source.x += terrain->width * terrain->world_scale;
  if (camera->source.z < 0)
    camera->source.z += terrain->height * terrain->world_scale;

  if (camera->source.x >= terrain->width * terrain->world_scale)
    camera->source.x -= terrain->width * terrain->world_scale;
  if (camera->source.z >= terrain->height * terrain->world_scale)
    camera->source.z -= terrain->height * terrain->world_scale;

  EntityCoordinateFix (spaceship_entity);
  PR_PositionCameraTarget (camera,
                           spaceship_entity->orientation.location.x,
                           spaceship_entity->orientation.location.y,
                           spaceship_entity->orientation.location.z);

  PR_SetActiveCamera (camera);
}


void InitializeMovement (void)
/* Sets the movement parameters for friction, speeds, camera movements */
{
  /* These are globals defined the PR input routines */
  TRANSLATE_SPEED    = 10;
  MAX_TRANSLATE_SPEED= 50;
  TRANSLATE_DRAG     = 5;

  PITCH_SPEED        = 0.2;
  MAX_PITCH_SPEED    =   2;
  PITCH_DRAG         = 0.1;

  YAW_SPEED          = 0.4;
  MAX_YAW_SPEED      =   2;
  YAW_DRAG           = 0.2;

  ROLL_SPEED         = 4;
  MAX_ROLL_SPEED     = 40.0f;
  ROLL_DRAG          = 2;

  currentspeed = 0;
  pitchspeed = 0;
  yawspeed = 0;
  rollspeed = 0;
  spinspeed = 0;

  /* These are some custom movement parameters */
  TILT_SPEED         = 0.2;
  MAX_TILT_SPEED     = 2;
  TILT_DRAG          = 0.1;
}



void main (int argc, char *argv[])
{
PR_DWORD move_count;

  printf ("Power Render Terrain Demo\n");
  printf ("Keys: A/Z   - Move forward and backward\n");
  printf ("      Space - Fire Missiles\n");
  printf ("Use the mouse to steer\n");
 
  ChooseDevice ();

  PR_Initialize (7500);                /* Maximum triangles per frame */

  if ((device == DEVICE_VGA) || (device == DEVICE_SVGA))
    {
     #ifndef WIN32
       wsetmode (3);                        /* Go back to text mode */
     #endif

     PR_FatalError ("This example requires a 3D card", "EX05");
    }


  InitializeInputDevices ();

  InitializeDevices ();

  terrain_size = 16;

  Initialize3DEngine ();


  PRGUI_GoStartPath ();
  InitializeAudio ();

  setlib ("space.prl");
  InitializeTexture ();

  LoadGameData ();
  LoadEnemyData ();


  InitializeWeapons ();
  InitializeEnemies ();

  SetInitialPosition (spaceship_entity);

  InitializeMovement ();

  /* -------------- Loop ----------- */
  PR_OpenScreen (PR_BACKBUFFER);

//  grSstControl (GR_CONTROL_DEACTIVATE);

  while (!kbdon[KEY_ESC])
    {
     move_count = animation_count;
     animation_count = 0;

     if (shoot_reload)
       shoot_reload -= move_count;

     /* -------------- Camera Position ----------- */
     UserInput ();

     #ifdef WIN32
       UpdateMessages ();
     #endif

     /* -------------- Render Normal Frame ----------- */

     #if defined (MSDD) || defined (WTDD)
       if (PR_LostSurface)
         PR_RestoreAll ();
     #endif

     PR_OpenScreen (PR_BACKBUFFER);

     PR_NewFrame ();

     PR_SetActiveCamera (camera);
     PR_DepthBufferFunction (PR_CMP_ALWAYS);
     PR_PositionEntity (sky_entity, camera->source.x, camera->source.y, camera->source.z);
     PR_TransformEntity (sky_entity);
     PR_RenderEntity (sky_entity);          
     PR_SetActiveCamera (camera);
     PR_DepthBufferFunction (PR_CMP_LEQUAL);

     PR_UpdateEmitters (move_count);     
     PR_UpdateParticles (move_count);     

     UpdateExplosions ();
     PR_TransformLights (&explosionlights);

     PR_UpdateTerrain (terrain);
     PR_TransformTerrain (terrain);

     PR_TransformLights (&sunlight);
     UpdateEnemies (move_count);

     if (camera_view > 1)
       {
        PR_TransformEntity (spaceship_entity);
        PR_RenderEntity (spaceship_entity);
       }

     PR_RotateSegment (ParticleEntities[SMOKE_PARTICLE], 0, 0.0f, 0.0f, -0.8f);
     PR_RotateSegment (ParticleEntities[FIRE_PARTICLE], 0, 0.0f, 0.0f, -0.8f);

 
     PR_RenderFrame ();

     PRSND_SetOrigin (camera->source.x, camera->source.y, camera->source.z);
     PRSND_UpdateVoices ();

/*     PR_OpenScreen (PR_BACKBUFFER);
     PRGUI_printf (10, 10, "FPS: %i", framerate);
     PRGUI_printf (10, 20, "Lights: %i", scenelights.NumLights);
     PRGUI_printf (10, 30, "Triangles Drawn: %i", PR_StatElementsDrawn ());
*/

     /* Calculate the frame rate */
     if (ticks > 30)
       {
        framerate = updates;
        ticks = 0;
        updates = 0;
       }
     else
       updates++; 

     PR_Flip (1);
    }

  mdeinit ();
  uninstallkbd ();
  wstoptimer ();
  wdonetimer ();


  CloseAudio ();

  PR_FreeCamera (camera);

  PR_CloseViewport (&viewport);

  PR_DeleteAllTextures ();
  PR_DeleteAllShadeTables ();
  PR_DeleteAllMaterials ();
  PR_ShutDown ();
}


