#include <math.h>
#include <malloc.h>
#include <stdlib.h>
#include <direct.h>
#include <conio.h>
#include <string.h>
#include <time.h>

#include <stdarg.h>

#include <pr.h>
#include <prgui.h>

#if defined (MSGLIDE) || defined (WTGLIDE)
#include <pr3dfx.h>
#include <glide.h>
#include <terrain.h>
#include <winutil.h>

#elif defined (WIN32)

#include <terrain.h>
#include <winutil.h>
#else

#include <terrain.h>

 #ifdef __3DFX__
  #include <pr3dfx.h>
  #include <glide.h>
 #endif
#endif

#include <prdev.h>

#include "lslib.h"

PR_DWORD AFOG_START = 6710;
PR_DWORD AFOG_END = 9490;




PR_CAMERA  * newcam;
PR_VIEWPORT  viewport;                  /* Our viewport structure */
PR_DWORD     vwidth, vheight;
PR_LIGHTLIST userlights;
PR_LIGHTLIST sunlight;

#define ABOVE_GROUND 250
#define MAX_HEIGHT 2000

color temppal[256];

PR_DWORD camframe = 0;

PR_OBJECT *sky_shape;
PR_ENTITY *sky_entity;

/* Frame Rate Control  */
PR_DWORD ticks = 0;             /* Total number of ticks passed */


void PR_TerrainWaveFunction (PR_WORD *col, PR_WORD *hgt, PR_WORD *shd,
                             PR_DWORD x, PR_DWORD y)
{
  return;       /* Ignore waves */
}


void timerproc (void)
{
  ticks++;
}                


void help_log_message (char *msg, ... )
{
FILE *                outfile;
va_list               arglist;

  outfile = fopen ("c:\\wgt95.log", "at");
  if (outfile == NULL)
    return;
  va_start (arglist, msg);
  vfprintf (outfile, msg, arglist);
  va_end (arglist);
  fprintf (outfile, "\n");
  fclose (outfile);
}



typedef struct {
  char idlength;
  char colormaptype;
  char imagetype;
  unsigned short colormapstart;
  unsigned short colormaplength;
  char colormapbits;
  unsigned short xstart, ystart, width, depth;
  char bits;
  char descriptor;
} PR_TGAHEADER;
PR_TGAHEADER tga;


void DumpScreenshot (void)
{
block pic;
FILE *out;
unsigned short *temp;
PR_UWORD rgb;
PR_DWORD l;
PR_DWORD r, g, b;
char outfile[80];
PR_DWORD number;


  number = 1;
  for (;;)
   {
    sprintf (outfile, "screen%i.tga", number);
    out = fopen (outfile, "rb");
    if (out == NULL)
      break;
    number++;
    fclose (out);
   }

  pic = PRGFX_NewBlock (0, 0, 639, 479);
  temp = (unsigned short *)pic;

  memset ((char *)&tga, 0, sizeof (PR_TGAHEADER));
  tga.descriptor = 0x20;
  tga.width = 640;
  tga.depth = 480;
  tga.idlength = 0;
  tga.imagetype = 2;
  tga.bits = 24;

  sprintf (outfile, "screen%i.tga", number);
  out = fopen (outfile, "wb");
  fwrite (&tga, sizeof (PR_TGAHEADER), 1, out);

  for (l = 0; l < 640*480; l++)
    {
     rgb = *temp++;

     r = (rgb >> 11);
     g = (rgb - (r << 11)) >> 5;
     b = (rgb - (r << 11) - (g << 5));

     fputc (b << 3, out);
     fputc (g << 2, out);
     fputc (r << 3, out);
    }

  fclose (out);

  free (pic);
}




PR_DWORD PR_HitLand (PR_TERRAIN *terrain, PR_REAL x, PR_REAL y, PR_REAL z)
{
PR_REAL hgt1;

 hgt1 = PR_GetTerrainHeight (terrain, x, z, terrain->level1_heightpic, terrain->level1_colorpic);

 if (y < hgt1)
   return 1;
 else
   return 0;
}


/* ---------------------------------------------------------------------- */
/* Makes sure the camera is above the landscape for the first frame */
/* ---------------------------------------------------------------------- */
void SetInitialPosition (void)
{
PR_REAL hgt;


  hgt = PR_GetTerrainHeight (terrain, newcam->source.x,
                                      newcam->source.z,
                                      terrain->level1_heightpic,
                                      terrain->level1_colorpic)
                                      + ABOVE_GROUND;

  PR_PositionCameraSource (newcam,
                      newcam->source.x,
                      hgt + ABOVE_GROUND,
                      newcam->source.z);
}



/* ---------------------------------------------------------------------- */
/* Checks for ground collision in the flying model */
/* ---------------------------------------------------------------------- */
void SetFlyingHeight (void)
{
PR_REAL hgt1;

  hgt1 = PR_GetTerrainHeight (terrain, newcam->source.x,
                                       newcam->source.z,
                                       terrain->level1_heightpic,
                                       terrain->level1_colorpic)
                                       + ABOVE_GROUND;

  if (PR_InputModel == MODEL_WALKING)
     newcam->source.y = hgt1;
  else  /* MODEL_FLYING */
    {
     if (newcam->source.y < hgt1)
       {
        newcam->dest.y += hgt1 - newcam->source.y;
        newcam->source.y = hgt1;
       }

/*     if (newcam->source.y > hgt1 + MAX_HEIGHT)
       {
        newcam->dest.y += (hgt1 + MAX_HEIGHT) - newcam->source.y;
        newcam->source.y = hgt1 + MAX_HEIGHT;
       }*/
    }
}







/* ---------------------------------------------------------------------- */
/* Check the mouse and keyboard input */
/* ---------------------------------------------------------------------- */
void UserInput (void)
{
  PR_ReadInput ();
  /* Move around depending on what movement model we are using */

  PR_CameraDirection (newcam);

  PR_DollyCamera (newcam, currentspeed);
  SetFlyingHeight ();

  if (newcam->source.x < 0)
    newcam->source.x += terrain->width * terrain->world_scale;
  if (newcam->source.z < 0)
    newcam->source.z += terrain->height * terrain->world_scale;

  if (newcam->source.x >= terrain->width * terrain->world_scale)
    newcam->source.x -= terrain->width * terrain->world_scale;
  if (newcam->source.z >= terrain->height * terrain->world_scale)
    newcam->source.z -= terrain->height * terrain->world_scale;

  PR_CameraDirection (newcam);

  PR_SetActiveCamera (newcam);
  /* Updates the transformation matrix */

//  if (kbdon[KEY_S])
//    DumpScreenshot ();

  if (kbdon[KEY_F1])
    {
     PR_SetInputModel (MODEL_WALKING);

     TRANSLATE_SPEED     = 40.0f;
     MAX_TRANSLATE_SPEED = 100.0f;
     TRANSLATE_DRAG      = 20.0f;

     PITCH_SPEED        = 0.6f;
     MAX_PITCH_SPEED    = 4.0f;
     PITCH_DRAG         = 0.2f;
    }
  else if (kbdon[KEY_F2])
    {
     PR_SetInputModel (MODEL_FLYING);

     TRANSLATE_SPEED     = 40.0f;
     MAX_TRANSLATE_SPEED = 100.0f;
     TRANSLATE_DRAG      = 20.0f;

     PITCH_SPEED        = 1.2f;
     MAX_PITCH_SPEED    = 8.0f;
     PITCH_DRAG         = 0.4f;
    }



/*  if (kbdon[KEY_Q])
    active_viewport.xscale += 0.05f;

  if (kbdon[KEY_W])
    active_viewport.xscale -= 0.05f;

  if (kbdon[KEY_A])
    active_viewport.centerx += 0.05f;

  if (kbdon[KEY_S])
    active_viewport.centerx -= 0.05f;
*/

}


void main (int argc, char *argv[])
{

//  diagnostics_level = 2;

  ChooseDevice ();

  PRGUI_InitPath (argv[0]);

  PR_Initialize (7500);                /* Maximum triangles per frame */

#ifndef WIN32
  vga256 ();
  minit ();
#endif
  msetbounds (0, 0, 319, 199);
  msetxy (160, 100);
  /* Necessary for the standard input routines */

  installkbd ();
  winittimer ();
  wstarttimer (timerproc, TICKS(60));

  InitializeDevices ();

  PR_AllocMaterials (256 * 6);
  PR_AllocTextures (256 * 6);
  PR_AllocShadeTables (32);

  vwidth = PR_VideoModes[0].width;     /* Initialize the video mode */
  vheight = PR_VideoModes[0].height;

help_log_message ("1");
  vwidth = 640;
  vheight = 480;
  PR_SetMode (vwidth, vheight, 60);
help_log_message ("2");

  PR_OpenViewport (&viewport, 0, 0, vwidth-1, vheight-1, VIEW_PLAIN);
  PR_SetViewport (&viewport);          /* Open a full screen viewport */
help_log_message ("3");

  PR_Settings.HardwareMipmaps = 1;

#if defined (MSGLIDE) || defined (WTGLIDE)
  PRGUI_GoStartPath ();
#endif

help_log_message ("4");
  PR_SetTextureFormat (TEXTURE_P_8);
help_log_message ("5");

  chdir ("projects");
  chdir ("swamp");
//  sprintf (temp, "%s.lsl", argv[1]);
  grSstControl (GR_CONTROL_DEACTIVATE);

help_log_message ("6");
  LS_LoadTerrain ("swamp.lsl");
help_log_message ("7");

  /* -------------- Misc Setup ----------- */
  gui_font = NULL;
  PRGFX_SetTextForeground (PRGFX_MakeColor (63,63,63));
  PRGFX_SetTextBackground (PRGFX_MakeColor (32,32,32));
  PRGFX_SetTextTransparent (TEXTFGBG);

help_log_message ("8");

  newcam = PR_AllocCamera ();
  PR_InitializeCamera (newcam);
  PR_PositionCameraSource (newcam, 0, 0, 0);
  PR_PositionCameraTarget (newcam, 0, 0, 0);
  PR_SetCameraMode (newcam, CAMFLAG_ANGLE_BASED);
  newcam->fov = 1.5f / 3.0f * (PR_REAL)PR_PI;
  PR_SetActiveCamera (newcam);
  SetInitialPosition ();
help_log_message ("9");

  PR_AllocLights (&userlights, 60);      /* User lights */
  PR_AllocLights (&sunlight, 1);         /* Infinite sun light */
  PR_AllocLights (&scenelights, 60);     /* Master scene light list */

  PR_SetLightPosition (&sunlight, 0, 5000, 100000, 50000);
  PR_SetLightType (&sunlight, 0, DIRECTIONAL_LIGHT);
  PR_SetLightOn (&sunlight, 0);
  PR_SetLightStrength (&sunlight, 0, 1.0);
  PR_SetLightColor (&sunlight, 0, 1.0, 1.0, 1.0);

  PR_SetLightType (&userlights, 0, POINT_LIGHT);
  PR_SetLightOn (&userlights, 0);
  PR_SetLightStrength (&userlights, 0, 1.0);
  PR_SetLightFalloff (&userlights, 0, 768);
  PR_SetLightColor (&userlights, 0, 1.0, 1.0, 1.0);

  /* Initialize the input devices */
  PR_SetInputDevice (INPUT_MOUSE);
  PR_SetInputModel (MODEL_WALKING);
  PR_SetInputEntity (NO_ENTITY, NO_SEGMENT);
  PR_SetInputCamera (newcam);

  TRANSLATE_SPEED     = 40.0f;
  MAX_TRANSLATE_SPEED = 100.0f;
  TRANSLATE_DRAG      = 20.0f;

  PITCH_SPEED        = 0.6f;
  MAX_PITCH_SPEED    = 4.0f;
  PITCH_DRAG         = 0.2f;

  userlights.NumLights = 1;     /* This sets how many lights are actually used */
  sunlight.NumLights = 1;     
help_log_message ("10");

  PRGUI_GoStartPath ();
  PR_SetTextureFormat (TEXTURE_P_8);
  sky_shape = PR_LoadPRO ("skycube.pro", LOAD_NORMAL);
  PR_SetTextureFormat (TEXTURE_NORMAL);
  sky_entity = PR_CreateEntity (sky_shape, "Sky");
  PR_PositionEntity (sky_entity, 0, 0, 0);
  PR_ScaleEntityAbs (sky_entity, 160, 160, 160);

  PRGFX_SetTextForeground (PRGFX_MakeColor (63, 63, 63));
  PRGFX_SetTextBackground (PRGFX_MakeColor (0, 0, 0));
  PRGFX_SetTextTransparent (TEXTFGBG);


#if !defined (MSGLIDE) && !defined (WTGLIDE)
  if ((device == DEVICE_VGA) || (device == DEVICE_SVGA))
    PR_SetPerspectiveDivisions (TEXTURE_DIVISION_16);
#endif

  /* -------------- Loop ----------- */
  PR_OpenScreen (PR_BACKBUFFER);

help_log_message ("11");

  while (!kbdon[KEY_ESC])
    {
     #ifdef WIN32
       UpdateMessages ();
     #endif

help_log_message ("12");

     UserInput ();
help_log_message ("13");

     /* -------------- Render Normal Frame ----------- */
     #if defined (MSDD) || defined (WTDD)
       if (PR_LostSurface)
         PR_RestoreAll ();
     #endif

     PR_OpenScreen (PR_BACKBUFFER);
     PR_NewFrame ();

help_log_message ("14");

     PR_SetActiveCamera (newcam);
     PR_SetFogState (FALSE);
     PR_DepthBufferFunction (PR_CMP_ALWAYS);
     PR_PositionEntity (sky_entity, newcam->source.x, newcam->source.y, newcam->source.z);
     PR_TransformEntity (sky_entity);
     PR_RenderEntity (sky_entity);          
     PR_DepthBufferFunction (PR_CMP_LEQUAL);
help_log_message ("15");

     PR_SetFogRange (AFOG_START, AFOG_END);


//     PRGFX_SetColor (Terrain_Settings.FogColor);
//     PRGFX_ClearScreen ();

help_log_message ("16");
     PR_UpdateTerrain (terrain);
help_log_message ("17");
     PR_TransformTerrain (terrain);
help_log_message ("18");

     PR_TransformLights (&sunlight);
help_log_message ("19");

     LS_RenderObjects ();
help_log_message ("20");

     PR_RenderFrame ();
help_log_message ("21");

/*     if (kbdon[KEY_T])
       AFOG_START -= 10;
     if (kbdon[KEY_G])
       AFOG_START += 10;
     if (kbdon[KEY_Y])
       AFOG_END -= 10;
     if (kbdon[KEY_H])
       AFOG_END += 10;

     PRGUI_printf (0, 0, "%i %i", AFOG_START, AFOG_END);
*/     

help_log_message ("22");
     PR_Flip (1);
    }

  mdeinit ();
  uninstallkbd ();
  wstoptimer ();
  wdonetimer ();

  PR_FreeCamera (newcam);

  PR_CloseViewport (&viewport);

  PR_DeleteAllTextures ();
  PR_DeleteAllShadeTables ();
  PR_DeleteAllMaterials ();
  PR_ShutDown ();
}

