#include <stdlib.h>
#include <stdarg.h>
#include <stdio.h>
#include <assert.h>

#include <pr.h>

#if defined(MSGLIDE) || defined (WTGLIDE)
#include <glide.h>
#elif defined (MSDD) || defined (WTDD)
#include <wgt95.h>
#endif

#include <dinput.h>

#define WIN32_LEAN_AND_MEAN
#include <windows.h>
#include <windowsx.h>
#include "winutil.h"


#define TITLE "Power Render 2.6"

/* Forward declarations */
HWND hWndMain;
HINSTANCE  ghInstance;    /* My instance handle */

extern int PR_WindowHandle;

static unsigned int AppTimer = 1;

/* The following variables are used to simulate the input routines
   from WGT 5.1 */
char kbdon[256];

PR_MouseCallback_data mouse;
int PR_mx0, PR_my0, PR_mx1, PR_my1;

int PR_AppActive;

int minit (void) { return 1;}
void mdeinit (void) {}
void msetbounds (int x0, int y0, int x1, int y1) {}
void msetxy (int x0, int y0) {}
void noclick (void) {}
void PR_ReadMouse (void) {}
void PR_MouseSyncAcquire (HWND hwnd) {}
int installkbd (void) { return 1;}
void PR_ReadKeyboard(void) {}
void uninstallkbd (void) { }
BOOL PR_KeyboardSyncAcquire (void) { return 1; }
char PR_getch (void) {  return 0; }
int PR_kbhit (void) { return 0; }
void UpdateMessages (void) {  }

extern void AppOpen (void);
extern void AppUpdate (void);
extern void AppClose (void);


void ResizeWindow (int width, int height)
{
  if (active_viewport.width > 0)
   {
    PR_ResizeViewport (&active_viewport, 0, 0, width-1, height-1);

    PRGFX_Clip (0, 0, width-1, height-1);
   }
  /* Need this so clipping is done correctly */
  PR_OutputDevice.width = width;
  PR_OutputDevice.height = height;

  if (PR_CurrentCamera != NULL)
    PR_CurrentCamera->aspect_ratio = (PR_REAL)width / (PR_REAL)height;
}




long FAR PASCAL MainWndproc( HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam )
{
extern void getWindowSize(float *width, float *height);
float width, height;

  switch (message)
    {
    case WM_CREATE:
      if (!SetTimer (hWnd, AppTimer, 15, NULL))
        PR_FatalError ("No timers available", "vrush");
        break;

    case WM_TIMER:
      if (PR_AppActive)
        AppUpdate ();
      break;

     case WM_SETCURSOR:
       if (!PR_Settings.UseWindowed)
         {
          SetCursor(NULL);
          return 0;
         }
       break;

     case WM_KEYDOWN:
       break;

     case WM_KEYUP:
         break;

    case WM_ACTIVATE:
       break;

    case WM_ENTERMENULOOP:
    case WM_ENTERSIZEMOVE:
         PR_AppActive = FALSE;
         break;

    case WM_EXITMENULOOP:
    case WM_EXITSIZEMOVE:
         PR_AppActive = GetActiveWindow() == hWnd;
         break;

               
    case WM_PAINT:
       break;

    case WM_CLOSE:
        break;

    case WM_DESTROY:
		KillTimer (hWnd, AppTimer);        
		PostQuitMessage (0);
        break;

    case WM_MOVE:
		break;

    case WM_DISPLAYCHANGE:
    case WM_SIZE:
      if (wParam != SIZE_MINIMIZED)
        {
         static int moving = 0;
         if (!moving)
           {
            if(!grSstControl(GR_CONTROL_RESIZE))
              {
               moving = 1;
               SetWindowPos (hWnd, 0, 0, 0, 300, 300, SWP_NOMOVE|SWP_NOZORDER);
               moving = 0;
               if(!grSstControl(GR_CONTROL_RESIZE))
                 {
                  PostMessage(hWnd,WM_CLOSE,0,0);
                 }
              }
            }


         getWindowSize(&width, &height);
         ResizeWindow (width, height);
        }
       break;


    default:
        break;
    }
    return DefWindowProc(hWnd, message, wParam, lParam);

} /* MainWndproc */

/*
 * initApplication
 *
 * Do that Windows initialization stuff...
 */
static BOOL initApplication( HANDLE hInstance, int nCmdShow )
{
    WNDCLASS    wc;
    BOOL        rc;

    ghInstance = hInstance;

    wc.style = CS_HREDRAW | CS_VREDRAW;
    wc.lpfnWndProc = MainWndproc;
    wc.cbClsExtra = 0;
    wc.cbWndExtra = 0;
    wc.hInstance = ghInstance;
    wc.hIcon = LoadIcon( NULL, IDI_APPLICATION);    /* generic icon */
    wc.hCursor = LoadCursor( NULL, IDC_ARROW );
    wc.hbrBackground = GetStockObject( BLACK_BRUSH );
    wc.lpszMenuName =  NULL;
    wc.lpszClassName = "PRClass";
    rc = RegisterClass( &wc );
    if( !rc )
    {
        return FALSE;
    }

    hWndMain = CreateWindowEx(
        WS_OVERLAPPEDWINDOW, 
        "PRClass",
        TITLE,
        WS_OVERLAPPEDWINDOW,      
        CW_USEDEFAULT, 
        CW_USEDEFAULT,
        320,                /* GetSystemMetrics(SM_CXSCREEN), */
        200,                /* GetSystemMetrics(SM_CYSCREEN), */
        NULL,
        NULL,
        hInstance,
        NULL );

    if (!hWndMain)
      return FALSE;

    if (!minit ()) 
       {
        DestroyWindow(hWndMain);
        return 0;
       }

    PR_WindowHandle = (int)hWndMain;

    ShowWindow (hWndMain, SW_NORMAL);
    UpdateWindow (hWndMain);
    
    BringWindowToTop (hWndMain);
    SetFocus (hWndMain);
        
    PR_AppActive = TRUE;
    PR_MouseSyncAcquire (hWndMain);
    return TRUE;
} /* initApplication */

/*
 * WinMain
 */
int PASCAL WinMain( HINSTANCE hInstance, HINSTANCE hPrevInstance, 
    LPSTR lpCmdLine, int nCmdShow )
{
MSG msg;

  if( !initApplication(hInstance, nCmdShow) )
    return FALSE;

  AppOpen ();
  while (GetMessage(&msg, NULL, 0, 0)) 
    {
     TranslateMessage(&msg);
     DispatchMessage(&msg);      
    }

  AppClose ();
  return 0;

} /* WinMain */





void getWindowSize(float *width, float *height)
{
RECT    rect;

  if (hWndMain != NULL)
    {
     GetClientRect(hWndMain, &rect);
     *width = (float) rect.right;
     *height = (float) rect.bottom; 
    }
  else
   {
    *width = 40;
    *height = 40;
   }
}






