#include <stdlib.h>
#include <direct.h>
#include <conio.h>
#include <string.h>
#include <ctype.h>
#include <malloc.h>

#include <stdarg.h>

#include <pr.h>
#include <prstate.h>

#if defined (MSGLIDE) || defined (WTGLIDE)
 #include <glide.h>
 #include <pr3dfx.h>
 #include "winutil.h"
 #include <windows.h>
#elif defined (WIN32)
 #include "winutil.h"
 #include <windows.h>
#else
 #ifdef __3DFX__
  #include <glide.h>
  #include <pr3dfx.h>
  #include <fxgui.h>
 #endif
#endif

#include <prgui.h>

#include <prdev.h>

/*
   PRLight 1.0

*/



/* --- Defines --- */

#define DISPWIDTH 480
#define DISPHEIGHT 470


/* --- Global Variables --- */

/* Variables read from the .ini file */
PR_REAL MOVEMENT_MULT=20;
PR_DWORD MAXLIGHT=500;
PR_REAL MAXRADIUS=15000;
PR_REAL POINTER_SCALE=200.0f;
PR_REAL FOG_START=2000.0f;
PR_REAL FOG_END=22000.0f;

/* Strings */
char scene_name[80];                  /* Name of the scene file (.PRO) */
char light_name[80];                  /* Name of the light file (.LGT) */

/* Menu Images */
block menu1;
color menu_pal[256];
PR_DWORD checktex;                    /* Texture numbers */
PR_DWORD unchecktex;
PR_DWORD slidertex;
PR_DWORD menu2tex;
PR_DWORD menu3tex;

/* Colors */
PR_DWORD BLACK;
PR_DWORD DARK_GREY;
PR_DWORD WHITE;

/* Objects and Entities */
PR_OBJECT *object;
PR_ENTITY *scene;
PR_OBJECT *pointer_object;
PR_ENTITY *pointer_entity;

/* Materials */
PR_DWORD point_mat;
PR_MATERIAL *white_material;

/* Cameras and Viewports */
PR_VIEWPORT viewport;
PR_CAMERA *camera;

/* Vectors */
PR_POINT UpVec;
PR_POINT ForwardVec;
PR_POINT RightVec;
PR_POINT TempVec;

/* Misc */
PR_DWORD start_screen, end_screen;
PR_DWORD mouse_savex = 0, mouse_savey = 0;
PR_DWORD flashflag = 0;                    /* Toggles between 0 and 1 */
PR_DWORD flashswitch = 0;                  /* True if flashing enabled */
PR_DWORD use_background;


/* Lights */
typedef struct {
  PR_DWORD active;

  PR_DWORD type;
  PR_POINT location;
  PR_REAL r, g, b, strength, radius;
} LightData;
LightData *MyLights;

PR_DWORD MaxLightUsed = 0;
PR_DWORD current_light = 0;
PR_LIGHTLIST userlights;            /* We only use 1 light */



/* Screen capture */
typedef struct {
  char idlength;
  char colormaptype;
  char imagetype;
  unsigned short colormapstart;
  unsigned short colormaplength;
  char colormapbits;
  unsigned short xstart, ystart, width, depth;
  char bits;
  char descriptor;
} PR_TGAHEADER;
PR_TGAHEADER tga;



void DumpScreenshot (void)
/* Saves a TGA screenshot.  Shots are automatically numbered */
{
block pic;
FILE *out;
unsigned short *temp;
PR_UWORD rgb;
PR_DWORD l;
PR_DWORD r, g, b;
char outfile[80];
PR_DWORD number;


  number = 1;
  for (;;)
   {
    sprintf (outfile, "screen%i.tga", number);
    out = fopen (outfile, "rb");
    if (out == NULL)
      break;
    number++;
    fclose (out);
   }

  pic = PRGFX_NewBlock (0, 0, 639, 479);
  temp = (unsigned short *)pic;

  memset ((char *)&tga, 0, sizeof (PR_TGAHEADER));
  tga.descriptor = 0x20;
  tga.width = 640;
  tga.depth = 480;
  tga.idlength = 0;
  tga.imagetype = 2;
  tga.bits = 24;

  sprintf (outfile, "screen%i.tga", number);
  out = fopen (outfile, "wb");
  fwrite (&tga, sizeof (PR_TGAHEADER), 1, out);

  for (l = 0; l < 640*480; l++)
    {
     rgb = *temp++;

     r = (rgb >> 11);
     g = (rgb - (r << 11)) >> 5;
     b = (rgb - (r << 11) - (g << 5));

     fputc (b << 3, out);
     fputc (g << 2, out);
     fputc (r << 3, out);
    }

  fclose (out);

  free (pic);
}



void LoadINI (void)
/* Loads the edit parameters from the INI file.
   Assumes a specific order for the contents of the file.
 */
{
FILE *ini;
char temp[80];
char buf[81];

  PRGUI_GoStartPath ();

  ini = fopen ("prlight.ini", "rt");
  if (ini == NULL)
    PR_FatalError ("Cannot find prlight.ini!", "PRLIGHT");

  fgets (buf, 80, ini);
  sscanf (buf, "%s %i", temp, &MAXLIGHT);
  
  fgets (buf, 80, ini);
  sscanf (buf, "%s %f", temp, &MOVEMENT_MULT);

  fgets (buf, 80, ini);
  sscanf (buf, "%s %f", temp, &MAXRADIUS);

  fgets (buf, 80, ini);
  sscanf (buf, "%s %f", temp, &POINTER_SCALE);

  fgets (buf, 80, ini);
  sscanf (buf, "%s %f", temp, &FOG_START);

  fgets (buf, 80, ini);
  sscanf (buf, "%s %f", temp, &FOG_END);

  fclose (ini);
}



void nokey (PR_DWORD keynum)
/* Waits for a specific key to be released */
{
PR_DWORD dummy = 0;

 while (kbdon[keynum])
   {
    #ifdef WIN32
      UpdateMessages ();
    #endif
    dummy++;
   }
}




void FindMaxLight (void)
/* Determines the index of the last light used */
{
PR_DWORD i;

  MaxLightUsed = 0;
  for (i = 0; i < MAXLIGHT; i++)
     if (MyLights[i].active)
       MaxLightUsed = i;
}


void InitializeLights (void)
/* Allocate room for the lights and initialize them to some default values. */
{
PR_DWORD i;
PR_MATERIAL *m;

  MyLights = (LightData *)malloc (sizeof (LightData) * MAXLIGHT);
  if (MyLights == NULL)
    exit (1);


  for (i = 0; i < MAXLIGHT; i++)
    {
     MyLights[i].active = 0;
     MyLights[i].r = 255.0f;
     MyLights[i].g = 255.0f;
     MyLights[i].b = 255.0f;
     MyLights[i].strength = 255.0f;
     MyLights[i].radius = 10.0f;
     MyLights[i].type = DIRECTIONAL_LIGHT;

     MyLights[i].location.x = 0;
     MyLights[i].location.y = 0;
     MyLights[i].location.z = 0;

    }

  PR_AllocLights (&userlights, MAXLIGHT);    /* User lights */
  PR_AllocLights (&scenelights, MAXLIGHT+1); /* Master scene light list */
  userlights.NumLights = MaxLightUsed;     /* This sets how many lights are actually used */

  /* Create a material for the actual light markers */
  m = &PR_ObjectMaterialList[0];
  white_material = m;
  PR_SetMaterialName (m, "white mat");
  PR_SetMaterialMethod (m, T_CFLAT);
  PR_SetMaterialColor (m, 255, 255, 255);
  PR_SetMaterialShades (m, 32);
}



void ComputeDirections (void)
/* Computes the forward and right direction vectors for moving the lights
   around */
{
  PR_CameraDirection (camera);
  ForwardVec.x = camera->direction.x;
  ForwardVec.y = camera->direction.y;
  ForwardVec.z = camera->direction.z;
  PR_VecCrossProduct(RightVec, ForwardVec, UpVec);
  ForwardVec.y = 0;
  RightVec.y = 0;
  PR_Normalize (&ForwardVec);
  PR_Normalize (&RightVec);
}



void LoadLights (char *filename)
/* Load the light values from a file.
   The camera position is also loaded. */
 {
PR_DWORD numlights;
FILE *in;

  PRGUI_GoUserPath ();

  in = fopen (filename, "rb");
  if (in == NULL)
    return;

  fread (&numlights, 4, 1, in);
  fread (MyLights, sizeof (LightData), numlights, in);

  fread (&camera->source.x, 4, 1, in);
  fread (&camera->source.y, 4, 1, in);
  fread (&camera->source.z, 4, 1, in);
  fread (&camera->rotation.x, 4, 1, in);
  fread (&camera->rotation.y, 4, 1, in);
  fread (&camera->rotation.z, 4, 1, in);
  ComputeDirections ();
  fclose (in);
}


void SaveLights (char *filename)
/* Save the lights file */
{
PR_DWORD segnum;
PR_SEGMENT *seg;

PR_DWORD numlights;
FILE *out;

  PRGUI_GoUserPath ();

  FindMaxLight ();
  numlights = MaxLightUsed + 1;

  out = fopen (filename, "wb");

  fwrite (&numlights, 4, 1, out);
  fwrite (MyLights, sizeof (LightData), numlights, out);

  fwrite (&camera->source.x, 4, 1, out);
  fwrite (&camera->source.y, 4, 1, out);
  fwrite (&camera->source.z, 4, 1, out);
  fwrite (&camera->rotation.x, 4, 1, out);
  fwrite (&camera->rotation.y, 4, 1, out);
  fwrite (&camera->rotation.z, 4, 1, out);
  fclose (out);


  /* Make sure we remove the always visible flag */
  for (segnum = 0; segnum < object->num_segments; segnum++)
    {
     seg = &object->segment_list[segnum];
     seg->flags &= ~FLAG_ALWAYS_VISIBLE;
    }

  PR_SavePRO (scene_name, object, SAVE_USED_MATERIALS);
}



void RenderLights (void)
/* Displays the light marker objects */
{
PR_DWORD i;

  for (i = 0; i < MAXLIGHT; i++)
    {
     if (MyLights[i].active)
       {
        /* Change the material of the object to match the light color.
           This is possible because the rendering method isn't alpha blended
           and polygons are drawn immediately.  */
        PR_ObjectMaterialList[point_mat].r = MyLights[i].r * MyLights[i].strength / 255.0f;
        PR_ObjectMaterialList[point_mat].g = MyLights[i].g * MyLights[i].strength / 255.0f;
        PR_ObjectMaterialList[point_mat].b = MyLights[i].b * MyLights[i].strength / 255.0f;

        PR_PositionEntity (pointer_entity, MyLights[i].location.x, MyLights[i].location.y, MyLights[i].location.z); 

        PR_TransformEntity (pointer_entity);

        /* If the marker flashing is on, and this is the current light, don't
           render it. */
        if ((flashflag) && (flashswitch) && (i == current_light))
          continue;

        PR_RenderEntity (pointer_entity);
       }
    }
}



void BuildSceneLights (void)
/* This goes through our custom light structure and initializes the userlights
   structure to match them. */
   
{
PR_DWORD i;
PR_REAL oo256;

  FindMaxLight ();

  oo256 = 1.0f / 256.0f;        /* Faster than 4 divides per light below */

  for (i = 0; i <= MaxLightUsed; i++)
    {
     if (MyLights[i].active)
       PR_SetLightOn (&userlights, i);
     else
       PR_SetLightOff (&userlights, i);
     PR_SetLightPosition (&userlights, i, MyLights[i].location.x, MyLights[i].location.y, MyLights[i].location.z);
     PR_SetLightType (&userlights, i, MyLights[i].type);
     PR_SetLightStrength (&userlights, i, MyLights[i].strength * oo256);
     PR_SetLightColor (&userlights, i, MyLights[i].r * oo256, MyLights[i].g * oo256, MyLights[i].b * oo256);
     PR_SetLightFalloff (&userlights, i, MyLights[i].radius);
    }

  userlights.NumLights = MaxLightUsed+1;    /* This sets how many lights are actually used */
}



void ComputeLighting (void)
/* This is does most of the work.  It computes the precalculated RGB lighting
   for an entity */
{
PR_SEGMENT *seg;
PR_VERTEX *vert;
PR_VERTEX_DATA *vdata;
PR_FACE *faceptr;
PR_DWORD *rgb;
PR_DWORD segnum, facenum, vertnum;

  BuildSceneLights ();
  PR_TransformLights (&userlights);

  /* ADDPRIV_mat must be initialized to something */
  ADDPRIV_mat = white_material;

  for (segnum = 0; segnum < object->num_segments; segnum++)
    {
     seg = &object->segment_list[segnum];
     vert = seg->vertex_list;

     /* This makes sure that all segments will be transformed even if
        there are not in the viewing frustum.  Otherwise lights will
        be computed incorrectly depending on the camera position/angle */
     seg->flags |= FLAG_ALWAYS_VISIBLE;

     /* This computes the normal vectors from this segment to all the
        directional lights. */
     PR_SetSegmentLighting (scene, seg, &scene->segment_orientation[segnum]);

     for (vertnum = 0; vertnum < seg->num_vertices; vertnum++)
       {
        vdata = vert->vdata;
        #if defined (MSDD) || defined (WTDD)
          PR_CalcVertexLightingD3D (vert, vdata);
        #elif defined (MSGL) || defined (WTGL)
          PR_CalcVertexLightingGL (vert, vdata);
        #else
          PR_CalcVertexLighting3Dfx (vert, vdata);
        #endif
        /* Now the RGB values are stored in the rendering cache.
           Pack them into the red value of the vertex data 
           (any 32 bit field will do).  We do this so that
           faces sharing vertices only need the colors computed
           once. */

        /* Cast the red value to a dword instead of a real */
        rgb = (PR_DWORD *)&vdata->r;

        /* Pack the color into the dword */
        *rgb =  (PR_DWORD)ADDPRIV_r +
               ((PR_DWORD)ADDPRIV_g << 8) +
               ((PR_DWORD)ADDPRIV_b << 16);

        vert++;
       }

     /* Now go through each face, look up the color of each vertex,
        and place it in the facedata structure. */
     for (facenum = 0; facenum < seg->num_faces; facenum++)
       {
        faceptr = &seg->face_list[facenum];

        faceptr->face_data.col[0] = *(PR_DWORD *)&faceptr->vertex1->vdata->r;
        faceptr->face_data.col[1] = *(PR_DWORD *)&faceptr->vertex2->vdata->r;
        faceptr->face_data.col[2] = *(PR_DWORD *)&faceptr->vertex3->vdata->r;
       }
    }
}






void DisplaySlider (PR_DWORD x1, PR_DWORD y1,
                    PR_DWORD x2, PR_DWORD y2,
                    PR_DWORD height, PR_REAL max)
/* Given the 2D region of a slider, computes the position of the slider
   icon and displays it.  The background behind the slider is assumed to be 
   drawn already */
{
PR_DWORD i;
PR_REAL slidepos;

  PR_DepthBufferFunction (PR_CMP_ALWAYS);

  slidepos = y2 - ((PR_REAL)height/max) * (PR_REAL)(y2-y1);
  slidepos -= 3;

  for (i = 0; i < PR_OutputDevice.pages; i++)
    {
     PR_OpenScreen (i);
     PR_NewFrame ();
     PRGFX_Clip (0, 0, 639, 479);

     PRGFX_ResizeTexture (x1 - 1, slidepos,
                          x1 + 12,
                          slidepos + 8,
                          slidertex, 1);
     PR_RenderFrame ();
    }

  PR_DepthBufferFunction (PR_CMP_LEQUAL);
}




void UpdateSliders (void)
/* Update the positions of all the sliders */
{
PR_DWORD i;

  PR_DepthBufferFunction (PR_CMP_ALWAYS);

  for (i = 0; i < PR_OutputDevice.pages; i++)
    {
     PR_OpenScreen (i);
     PR_NewFrame ();
     PRGFX_Clip (0, 0, 639, 479);
     PRGFX_ResizeTexture (493, 238, 632, 315, menu3tex, NORMAL);
     PR_RenderFrame ();
    }
  PR_DepthBufferFunction (PR_CMP_LEQUAL);

  DisplaySlider (501, 245, 513, 309, MyLights[current_light].r, 255.0f);
  DisplaySlider (527, 245, 539, 309, MyLights[current_light].g, 255.0f);
  DisplaySlider (553, 245, 565, 309, MyLights[current_light].b, 255.0f);
  DisplaySlider (584, 245, 596, 309, MyLights[current_light].strength, 255.0f);
  DisplaySlider (614, 245, 626, 309, MyLights[current_light].radius, MAXRADIUS);

}


void DisplayType (void)
/* Show the check boxes for the light type */
{
PR_DWORD i;

  PRGUI_HideMouse ();
  PR_DepthBufferFunction (PR_CMP_ALWAYS);

  for (i = 0; i < PR_OutputDevice.pages; i++)
    {
     PR_OpenScreen (i);
     PR_NewFrame ();

     if (MyLights[current_light].active)
       {
        PRGFX_ResizeTexture (514, 176, 525, 187, unchecktex, NORMAL);
         
        if (MyLights[current_light].type == DIRECTIONAL_LIGHT)
          PRGFX_ResizeTexture (514, 196, 525, 207, checktex, NORMAL);
        else
          PRGFX_ResizeTexture (514, 196, 525, 207, unchecktex, NORMAL);

        if (MyLights[current_light].type == POINT_LIGHT)
          PRGFX_ResizeTexture (514, 216, 525, 227, checktex, NORMAL);
        else
          PRGFX_ResizeTexture (514, 216, 525, 227, unchecktex, NORMAL);
       }
     else
       {
        PRGFX_ResizeTexture (514, 176, 525, 187, checktex, NORMAL);
        PRGFX_ResizeTexture (514, 196, 525, 207, unchecktex, NORMAL);
        PRGFX_ResizeTexture (514, 216, 525, 227, unchecktex, NORMAL);
       }
     PR_RenderFrame ();
    }

  PR_DepthBufferFunction (PR_CMP_LEQUAL);

  PRGUI_ShowMouse ();
}



void UpdateLightNumber (void)
/* Display the current light number */
{
PR_DWORD i;

  PRGFX_SetTextForeground (BLACK);
  PRGFX_SetTextTransparent (TEXTFG);
  PRGFX_Clip (0, 0, 639, 479);

  for (i = 0; i < PR_OutputDevice.pages; i++)
    {
     PR_OpenScreen (i);
     PR_NewFrame ();
     PRGFX_ResizeTexture (508, 120, 633, 153, menu2tex, NORMAL);
     PR_RenderFrame ();
     PRGUI_printf (520, 133, "Light %i", current_light);
    }
}




void DisplayString (PR_DWORD x, PR_DWORD y, char *fmt, ...)
/* Displays a string in the status bar along the bottom of the screen */
{
va_list  argptr;                     /* Argument list pointer*/
char str[251];                       /* Buffer to build string into*/
PR_DWORD i;

  va_start (argptr, fmt);              /* Initialize va_ functions*/

  vsprintf (str, fmt, argptr);         /* prints string to buffer*/

  PRGFX_SetTextForeground (WHITE);
  PRGFX_SetTextBackground (DARK_GREY);
  PRGFX_SetTextTransparent (TEXTFGBG);
  PRGFX_SetColor (DARK_GREY);

  for (i = 0; i < PR_OutputDevice.pages; i++)
    {
     PR_OpenScreen (i);

     PRGFX_Bar (0, 471,  479, 479);
     PRGFX_OutTextXY (x, y, gui_font, str);
     /* Send string in graphics mode */
    }

  va_end (argptr);                     /* Close va_ functions*/
}


void InitMainMenu (void)
/* Draw the menu, plus misc setup */
{
PR_DWORD i;

  PRGUI_GoStartPath ();
  setlib ("prlight.gui");

  gui_font = wloadfont ("system.wfn");

  unchecktex = PR_LoadTexture("uncheck.pcx");
  checktex   = PR_LoadTexture("check.pcx");
  slidertex  = PR_LoadTexture("slider.pcx");

  menu1 = wloadpcx ("menu1.pcx", menu_pal);
  menu2tex = PR_LoadTexture ("menu2.pcx");
  menu3tex = PR_LoadTexture ("menu3.pcx");

  setlib (NULL);

  BLACK = PRGFX_MakeColor (0,0,0);
  DARK_GREY = PRGFX_MakeColor (20,20,20);
  WHITE = PRGFX_MakeColor (63,63,63);

  PRGUI_HideMouse ();

  for (i = 0; i < PR_OutputDevice.pages; i++)
    {
     PR_OpenScreen (i);

     PRGFX_SetColor (GUI_BLACK);
     PRGFX_ClearScreen ();
     
     PR_BeginScreen ();
     PRGFX_PutBlock8bit (480, 0, menu1, menu_pal, 0);
     PR_EndScreen ();
    }

  current_light = 0;


  UpdateSliders ();
  UpdateLightNumber ();

  DisplayType ();

  PRGUI_ShowMouse ();

  wfreeblock (menu1);

  PRGUI_ShowMouse ();

  DisplayString (3, 471, "Loading Scene");
}



void Init3D (void)
/* Initialize the 3D engine, camera, viewports, etc */
{
  PRGUI_HideMouse ();

  UpVec.x = 0;
  UpVec.y = 1;
  UpVec.z = 0;

  PRGUI_GoUserPath ();
  object = PR_LoadPRO (scene_name, LOAD_NORMAL);

  if (object == NULL)
    PR_FatalError ("Cannot open PRO scene", "PRLIGHT");

  scene = PR_CreateEntity (object, "scene");

  PRGUI_GoStartPath ();
  setlib ("prlight.gui");
  pointer_object = PR_LoadPRO ("pointer.pro", LOAD_NORMAL);
  setlib (NULL);
  
  pointer_entity = PR_CreateEntity (pointer_object, "pointer");
  point_mat = PR_FindMaterial("lightpointer");
  PR_ScaleEntity (pointer_entity, POINTER_SCALE, POINTER_SCALE, POINTER_SCALE);


  /* Open the 3D viewport */
  PR_OpenViewport (&viewport, 0, 0, DISPWIDTH-1, DISPHEIGHT-1, VIEW_PLAIN);

  PR_SetViewport (&viewport);

  camera = PR_AllocCamera ();
  PR_InitializeCamera (camera);
  PR_PositionCameraSource (camera, 0, 0, object->bbox.radius * 2);
  camera->rotation.y = 0;
  camera->aspect_ratio = 1.0;
  PR_SetCameraMode (camera, CAMFLAG_ANGLE_BASED);
  PR_SetActiveCamera (camera);
  ComputeDirections ();

  PR_SetInputDevice (INPUT_MOUSE);
  PR_SetInputModel (MODEL_INSPECT);
  PR_SetInputEntity (NO_ENTITY, NO_SEGMENT);
  PR_SetInputCamera (camera);

  PR_AmbientRed   = 0;
  PR_AmbientGreen = 0;
  PR_AmbientBlue  = 0;

  PRGFX_Clip (0, 0, 639, 479);
  PRGUI_ShowMouse ();
}


extern PR_DWORD last_method;

void Display3D (void)
/* Update the 3D window */
{
  flashflag = !flashflag;

  PR_SetActiveCamera (camera);

  PR_OpenScreen (PR_BACKBUFFER);
  PRGFX_Clip (active_viewport.topx,
              active_viewport.topy,
              active_viewport.bottomx,
              active_viewport.bottomy);

  PRGFX_SetColor (BLACK);
  PR_ClearViewport (&viewport);

  PR_NewFrame ();                         /* Begin a new frame */

  if (use_background)
    {
     PR_SetConstantZ (64000);
     PRGFX_ResizeTexture (0, 0, 639, 479, PR_FindTexture ("back.pcx"), 0);
     PR_SetConstantZ (1);
    }

  PR_TransformEntity (scene);
  PR_RenderEntity (scene);

  RenderLights ();
  
  PR_RenderFrame ();

  PRGFX_Clip (0, 0, 639, 479);

  PR_Flip (1);
}




void DisplayHelp (void)
/* Show the help screen */
{
  PRGFX_SetTextForeground (WHITE);
  PRGFX_SetTextTransparent (TEXTFG);

  PRGUI_printf (0, 20, "PRLight 1.0");
  PRGUI_printf (0, 30, "Copyright 1998 Egerter Software");
  PRGUI_printf (0, 40, "Visit http://www.egerter.com");

  PRGUI_printf (0, 60, "Hold down CTRL and slide mouse up and down to dolly the");
  PRGUI_printf (0, 70, "camera.");
  PRGUI_printf (0, 90, "Hold down ALT and move the mouse to tilt camera.");
  PRGUI_printf (0, 110, "Hold the left mouse button and drag the current light");
  PRGUI_printf (0, 120, "while in the 3D display area.");

  PRGUI_printf (0, 140, "Current light will flash while CTRL or ALT is pressed.");

  PRGUI_printf (0, 160, "HOME moves the camera to the origin.");
  PRGUI_printf (0, 170, "ENTER moves the camera to the current light.");
  PRGUI_printf (0, 180, "SPACE moves the current light to the camera.");
  PRGUI_printf (0, 190, "PGUP goes to previous light.");
  PRGUI_printf (0, 200, "PGDN goes to next light.");
  PRGUI_printf (0, 210, "ESC quits.");

  nokey (KEY_F1);
  Display3D ();
}




PR_REAL ComputeSlider (PR_DWORD x1, PR_DWORD y1,
                       PR_DWORD x2, PR_DWORD y2,
                       PR_DWORD height, PR_REAL max)
/* Given the 2D region of a slider, and the position of the mouse cursor,
   computes the new value from 0 to the max value given. */
{
PR_REAL dist;
PR_REAL length;
PR_REAL val;

  dist = y2 - height;
  length = y2 - y1;
  val = (dist / length) * max;

  PRGUI_HideMouse ();
  UpdateSliders ();

  PRGUI_ShowMouse ();
  return val;
}




void MainLoop (void)
{
PR_DWORD quit = 0;
PR_REAL dist;

  PRGUI_HideMouse ();
 
  Display3D ();         /* Transform the entity once to generate the transformed vertices */
  ComputeLighting ();   /* Compute the lighting based on these transformed vertices */
  Display3D ();         /* Display the results */
  PRGUI_ShowMouse ();

  UpdateSliders ();
  DisplayType ();

  while ((!quit) && (!kbdon[KEY_ESC]))
   {
    #ifdef WIN32
      UpdateMessages ();
    #endif

    #if defined (MSDD) || defined (WTDD)
      if (PR_LostSurface)
        PR_RestoreAll ();
    #endif


    /* Rotate Camera */    
    if (kbdon[KEY_ALT])
      {
       PRGUI_HideMouse ();
       mouse_savex = mouse.mx;
       mouse_savey = mouse.my;
       msetxy (160, 100);
       while (kbdon[KEY_ALT])
         {
          #ifdef WIN32
            UpdateMessages ();
          #endif
          PR_ReadInput ();
          msetxy (160, 100);

          flashswitch = 1;
          Display3D ();
         }

       if (flashflag)   /* Make sure light is on */
         Display3D ();
       msetxy (mouse_savex, mouse_savey);
       PRGUI_ShowMouse ();

       ComputeDirections ();
      }


    /* Dolly Camera */
    if (kbdon[KEY_CTRL])
      {
       PRGUI_HideMouse ();
       mouse_savex = mouse.mx;
       mouse_savey = mouse.my;
       msetxy (160, 100);
       while (kbdon[KEY_CTRL])
         {
          #ifdef WIN32
            UpdateMessages ();
          #endif
          PR_DollyCamera (camera, (100 - mouse.my) * MOVEMENT_MULT);
          msetxy (160, 100);

          flashswitch = 1;
          Display3D ();
         }

       if (flashflag)   /* Make sure light is on */
         Display3D ();
       msetxy (mouse_savex, mouse_savey);
       PRGUI_ShowMouse ();
      }

    if ((PRGUI_HitButton (494, 122, 509, 137)) || (kbdon[KEY_PGUP]))
    /* Light up */
      {
       if (current_light > 0)
         current_light--;

       UpdateSliders ();
       UpdateLightNumber ();
       DisplayType ();
       noclick ();
       nokey (KEY_PGUP);
      }

    if ((PRGUI_HitButton (494, 138, 509, 153)) || (kbdon[KEY_PGDN]))
    /* Light down */
      {
       if (current_light < MAXLIGHT-1)
         current_light++;

       UpdateSliders ();
       UpdateLightNumber ();
       DisplayType ();
       noclick ();
       nokey (KEY_PGDN);
      }

    if (PRGUI_HitButton (514, 176, 525, 187))
    /* Off */
      {
       MyLights[current_light].active = 0;
       DisplayType ();

       ComputeLighting ();
       Display3D ();

       noclick ();
      }

    if (PRGUI_HitButton (514, 196, 525, 207))
    /* Directional */
      {
       if (!MyLights[current_light].active)
         {
          MyLights[current_light].location.x = camera->source.x - camera->direction.x * MOVEMENT_MULT * 35;
          MyLights[current_light].location.y = camera->source.y - camera->direction.y * MOVEMENT_MULT * 35;
          MyLights[current_light].location.z = camera->source.z - camera->direction.z * MOVEMENT_MULT * 35;
         }

       MyLights[current_light].type = DIRECTIONAL_LIGHT;
       MyLights[current_light].active = 1;
       DisplayType ();

       ComputeLighting ();
       Display3D ();

       if (flashflag)   /* Make sure light is on */
         Display3D ();
       noclick ();
      }

    if (PRGUI_HitButton (514, 216, 525, 227))
    /* Point */
      {
       if (!MyLights[current_light].active)
         {
          MyLights[current_light].location.x = camera->source.x - camera->direction.x * MOVEMENT_MULT * 35;
          MyLights[current_light].location.y = camera->source.y - camera->direction.y * MOVEMENT_MULT * 35;
          MyLights[current_light].location.z = camera->source.z - camera->direction.z * MOVEMENT_MULT * 35;
         }

       MyLights[current_light].type = POINT_LIGHT;
       MyLights[current_light].active = 1;
       DisplayType ();
       ComputeLighting ();
       Display3D ();

       if (flashflag)   /* Make sure light is on */
         Display3D ();
       noclick ();
      }


    if (PRGUI_HitButton (501, 245, 513, 309))
    /* Red */
      {
       msetbounds (501, 245, 513, 309);
       while (mouse.but)
         {
              #ifdef WIN32
               UpdateMessages ();
                  #endif

          MyLights[current_light].r =
              ComputeSlider (501, 245, 513, 309, mouse.my, 255.0f);

          ComputeLighting ();
          Display3D ();

          DisplayString (3, 471, "Red: %7.2f", MyLights[current_light].r);
         }
       if (flashflag)   /* Make sure light is on */
         Display3D ();
       msetbounds (0, 0, 639 - 16, 479 - 16);
      }


    if (PRGUI_HitButton (528, 245, 540, 309))
    /* Green */
      {
       msetbounds (528, 245, 540, 309);
       while (mouse.but)
         {
              #ifdef WIN32
               UpdateMessages ();
                  #endif
          MyLights[current_light].g =
              ComputeSlider (528, 245, 540, 309, mouse.my, 255.0f);

          ComputeLighting ();
          Display3D ();

          DisplayString (3, 471, "Green: %7.2f", MyLights[current_light].g);
         }
       if (flashflag)   /* Make sure light is on */
         Display3D ();
       msetbounds (0, 0, 639 - 16, 479 - 16);
      }


    if (PRGUI_HitButton (554, 245, 566, 309))
    /* Blue */
      {
       msetbounds (554, 245, 566, 309);
       while (mouse.but)
         {
          #ifdef WIN32
           UpdateMessages ();
          #endif
          MyLights[current_light].b =
              ComputeSlider (554, 245, 566, 309, mouse.my, 255.0f);

          ComputeLighting ();
          Display3D ();

          DisplayString (3, 471, "Blue: %7.2f", MyLights[current_light].b);
         }
       if (flashflag)   /* Make sure light is on */
          Display3D ();
       msetbounds (0, 0, 639 - 16, 479 - 16);
      }


    if (PRGUI_HitButton (585, 245, 597, 309))
    /* Strength */
      {
       msetbounds (585, 245, 597, 309);
       while (mouse.but)
         {
          #ifdef WIN32
           UpdateMessages ();
          #endif
          MyLights[current_light].strength =
              ComputeSlider (585, 245, 597, 309, mouse.my, 255.0f);

          ComputeLighting ();
          Display3D ();
          DisplayString (3, 471, "Strength: %7.2f", MyLights[current_light].strength / 255.0f);
         }
       if (flashflag)   /* Make sure light is on */
         Display3D ();
       msetbounds (0, 0, 639 - 16, 479 - 16);
      }


    if (PRGUI_HitButton (615, 245, 627, 309))
    /* Radius */
      {
       msetbounds (615, 245, 627, 309);
       while (mouse.but)
         {
          #ifdef WIN32
           UpdateMessages ();
          #endif
          MyLights[current_light].radius =
              ComputeSlider (615, 245, 627, 309, mouse.my, MAXRADIUS);

          ComputeLighting ();
          Display3D ();
          DisplayString (3, 471, "Radius: %7.2f", MyLights[current_light].radius);
         }
       if (flashflag)   /* Make sure light is on */
         Display3D ();
       msetbounds (0, 0, 639 - 16, 479 - 16);
      }

    if (PRGUI_HitButton (483, 386, 636, 411))
    /* Save lights */
      {
       DisplayString (3, 471, "Saving Lights and PRO file");
       SaveLights (light_name);
       noclick ();
       DisplayString (3, 471, " ");
      }

    if (PRGUI_HitButton (483, 447, 636, 472))
      quit = 1;

   if (PRGUI_HitButton (0, 0, 479, 469))
   /* 3D Display */
     {
      PRGUI_HideMouse ();
      msetxy (160, 100);
      while (mouse.but)
        {
         #ifdef WIN32
           UpdateMessages ();
         #endif

         if ((mouse.my != 100) | (mouse.mx != 160))
           {
            if (mouse.but == 1)
              {
               dist = (mouse.my - 100) * MOVEMENT_MULT;
               PR_VecScalarMultiply(TempVec, dist, ForwardVec);
               PR_VecAdd(MyLights[current_light].location, MyLights[current_light].location, TempVec);

               dist = (mouse.mx - 160) * MOVEMENT_MULT;
               PR_VecScalarMultiply(TempVec, dist, RightVec);
               TempVec.y = 0;
               PR_VecAdd(MyLights[current_light].location, MyLights[current_light].location, TempVec);
              }
            else if (mouse.but == 2)
               MyLights[current_light].location.y -= (mouse.my - 100) * MOVEMENT_MULT;

            msetxy (160, 100);
            ComputeLighting ();

            flashswitch = 0;
            Display3D ();
            DisplayString (3, 471, "X: %7.2f  Y: %7.2f  Z: %7.2f", MyLights[current_light].location.x, MyLights[current_light].location.y, MyLights[current_light].location.z);
           }
         }
       PRGUI_ShowMouse ();
     }


    /* Extra keyboard controls */
    if (kbdon[KEY_S])
      DumpScreenshot ();

    if (kbdon[KEY_F1])
      DisplayHelp ();

    if (kbdon[KEY_HOME])        /* Center camera */
      {
       PR_PositionCameraSource (camera, 0, 0, 0);
       Display3D ();
      }

    if (kbdon[KEY_SPACE])        /* Move light to camera */
      {
       MyLights[current_light].location.x = camera->source.x - camera->direction.x * MOVEMENT_MULT * 35;
       MyLights[current_light].location.y = camera->source.y - camera->direction.y * MOVEMENT_MULT * 35;
       MyLights[current_light].location.z = camera->source.z - camera->direction.z * MOVEMENT_MULT * 35;
       Display3D ();
       if (flashflag)   /* Make sure light is on */
         Display3D ();
      }

    if (kbdon[KEY_ENTER])        /* Move camera to light */
      {
       camera->source.x = MyLights[current_light].location.x + camera->direction.x * MOVEMENT_MULT * 35;
       camera->source.y = MyLights[current_light].location.y + camera->direction.y * MOVEMENT_MULT * 35;
       camera->source.z = MyLights[current_light].location.z + camera->direction.z * MOVEMENT_MULT * 35;
       Display3D ();
       if (flashflag)   /* Make sure light is on */
         Display3D ();
      }

    PRGUI_DrawMouse ();
   }

  PRGUI_HideMouse ();
  noclick ();
}



void main (int argc, char *argv[])
{
  PRGUI_InitPath (argv[0]);

  LoadINI ();

  if (argc < 3)
    {
     PR_FatalError (
             "PRLight 1.0\n"
             "Copyright 1998 Egerter Software\n\n"
             "Usage: \n"
             "PRLIGHT scene.pro scene.lgt\n", "PRLIGHT");
    }


  strcpy (scene_name, argv[1]);
  strcpy (light_name, argv[2]);

  ChooseDevice ();

  PR_Initialize (2500);

  if ((device == DEVICE_VGA) || (device == DEVICE_SVGA))
    {
     #ifndef WIN32
      wsetmode (3);                        /* Go back to text mode */
     #endif
     PR_FatalError ("This utility requires a 3D card", "PRLIGHT");
    }

  PRGUI_GoStartPath ();

  InitializeDevices ();

  PR_AllocMaterials (512);
  PR_AllocShadeTables (32);
  PR_AllocTextures (512);

  setlib ("guidata.prl");
 
  ChooseGUI ();

  setlib (NULL);


  if (PR_OutputDevice.pages == 1)
    {
     start_screen = PR_FRONTBUFFER;
     end_screen = PR_FRONTBUFFER;
    }
  else 
    {
     start_screen = PR_FRONTBUFFER;
     end_screen = PR_BACKBUFFER;
    }

  PRGUI_GoUserPath ();
  setlib (NULL);

  PR_Settings.LoadPalette = 1;
  PR_Settings.HardwareMipmaps = -1;

  PRGUI_InitTimer ();
  installkbd ();

  use_background = PR_LoadTexture ("back.pcx");
  if (PR_WorldTextures[use_background].width > 0)
    use_background = 1;
  else
    use_background = 0;

  InitializeLights ();
  Init3D ();
  InitMainMenu ();

  LoadLights (light_name);
  PR_SetFogRange (FOG_START, FOG_END);

  DisplayString (3, 471, "Ready... Hold F1 for help");

  MainLoop ();

  uninstallkbd ();
  PRGUI_DeinitTimer ();

  PRGUI_GoUserPath ();
}


