

/* MDL/PAK structures (Thanks to Trey Harrison) */
typedef struct
{
  float x, y, z;
} vec3_t;

typedef struct
{
  long id;                     /* 0x4F504449 = "IDPO"                  */
  long version;                /* Version = 3                          */
  vec3_t scale;                /* Scale factor, for x,y,z              */
  vec3_t origin;               /* Model origin: point for x=0,y=0,z=0  */
  float radius;                /* Model radius, maybe useless now.     */
  vec3_t eye;                  /* Position of the eyes                 */
  long numskins ;              /* the number of skin textures          */
  long skinwidth;              /* Width of skin texture                */
                               /*           must be multiple of 8      */
  long skinheight;             /* Height of skin texture               */
                               /*           must be multiple of 8      */
  long numverts;               /* Number of vertices                   */
  long numtris;                /* Number of triangles surfaces         */
  long numframes;              /* Number of frames                     */
  long sync;                   /* Sync type                            */
  long flags;                  /* duh                                  */
  float size;                  /* dunno. pixels / triangle ??          */
} mdlheader_t;


typedef struct
{
  int ident;
  int version;
  int skinwidth;
  int skinheight;
  int framesize; // byte size of each frame
  int num_skins;
  int num_xyz;
  int num_st; // greater than num_xyz for seams
  int num_tris;
  int num_glcmds; // dwords in strip/fan command list
  int num_frames;
  int ofs_skins; // each skin is a MAX_SKINNAME string
  int ofs_st; // byte offset from start for stverts
  int ofs_tris; // offset for dtriangles
  int ofs_frames; // offset for first frame
  int ofs_glcmds; 
  int ofs_end; // end of file
} mdlheader_t2;


typedef struct
{
  long        type;            /* Group type(1) or regular(0)          */
  long        numgroupskins;   /* only used on group skins             */
  float       *interval;       /* only used on group skins             */
  unsigned char **skin;        /* the skin picture, length = skinwidth*skinheight */
} skin_t;                      /* numskins pictures                    */
   

typedef struct
{
  long onseam;                 /* 0 or 1                               */
  long s;                      /* position, horizontally               */
                               /*  in range [0,skinwidth]              */
  long t;                      /* position, vertically                 */
                               /*  in range [0,skinheight]             */
} stvert_t;


typedef struct
{
  long facesfront;             /* boolean                              */
  long vertices[3];            /* Index of 3 triangle vertices         */
                               /* in range [0,numverts]                */
} itriangle_t;


typedef struct
{
  unsigned char x;
  unsigned char y;             /* X,Y,Z coordinate, packed on 0-255    */
  unsigned char z;
  unsigned char lightnormalindex;  /* index of the vertex normal       */
} trivertx_t;


typedef struct
{
  trivertx_t min;               
  trivertx_t max;               
  char name[16];               /* name of frame                        */ 
} frameinfo_t;

typedef struct
{
  vec3_t scale; // multiply byte verts by this
  vec3_t origin; // then add this
  char name[16]; // frame name from grabbing
  trivertx_t verts[1]; // variable sized
} frameinfo_t2;

typedef struct { 
  long type;                   /* single frame(0) or group frame(1)    */
  long numgroupframes;         /* used for groups only, duh            */
  trivertx_t groupmin;         /* minimum values of X,Y,Z for group    */
  trivertx_t groupmax;         /* maximum values of X,Y,Z for group    */
  frameinfo_t *info;           /* info on each frame                   */
  float *interval;             /* interval crap, dunno what it is      */
  trivertx_t **data;           /* length of this array is numverts*sizeof(trivertx_t) */
} frame_t;

typedef struct
{
  char            classname[40];
  vec3_t          scale;
  vec3_t          origin;
  long            numskins;
  skin_t          *skins;
  long            numverts;
  stvert_t        *verts;
  long            numtris;
  itriangle_t     *triangles;
  long            numframes;
  frame_t         *frames;
  long            skinwidth;
  long            skinheight;
  long            scaledwidth;
  long            scaledheight;
} mdl_t;


typedef struct {
    char            **skinnames;
    long            num_skins;
    
    long            num_glcmds;
    long            *glcmds;
    
    long            numframes;
    frameinfo_t2    *frames;    
    long            framesize;

    long            skinwidth;
    long            skinheight;
		
    float           s_scale;
    float           t_scale;
} mdl_t2;



typedef struct
{
  unsigned char magic[4];      /* Name of the new WAD format = "PACK" */
  long diroffset;              /* Position of WAD directory from start of file */
  long dirsize;                /* Number of entries * 0x40 (64 char) */
} pakheader_t;

typedef struct
{
  unsigned char filename[0x38]; /* Name of the file, Unix style, with extension, */
                                /* 56 chars, padded with '\0'. */
  long offset;                  /* Position of the entry in PACK file */
  long size;                    /* Size of the entry in PACK file */
} pakentry_t;


PR_OBJECT *PR_ImportMDL (mdl_t *mdl);
mdl_t *PR_LoadMDLFromPak (char *fname,char *pakname);

PR_OBJECT *PR_ImportMD2 (mdl_t2 *mdl);
mdl_t2 *PR_LoadMD2FromPak (char *fname,char *pakname);

void FixMDLNormals (int numframes);

